@@ -7,6 +7,7 @@ import pkg_resources
 import xarray as xr
 
 from .. import utils
+from .. import __version__
 
 CoordSpec = Dict[str, List[Any]]
 DimSpec = Dict[str, List[str]]
@@ -18,7 +19,7 @@ class requires:  # pylint: disable=invalid-name
     If the decorator is called various times on the same function with different
     attributes, it will return None if one of them is missing. If instead a list
     of attributes is passed, it will return None if all attributes in the list are
-    missing. Both functionalities can be combines as desired.
+    missing. Both functionalities can be combined as desired.
     """
 
     def __init__(self, *props):
@@ -82,7 +83,7 @@ def generate_dims_coords(shape, var_name, dims=None, coords=None, default_dims=N
                 shape_len=len(shape),
                 defaults=",".join(default_dims) + ", " if default_dims is not None else "",
             ),
-            SyntaxWarning,
+            UserWarning,
         )
     if coords is None:
         coords = {}
@@ -142,7 +143,7 @@ def numpy_to_data_array(ary, *, var_name="data", coords=None, dims=None):
             "Passed array should have shape (chains, draws, *shape)".format(
                 n_chains=n_chains, n_samples=n_samples
             ),
-            SyntaxWarning,
+            UserWarning,
         )
 
     dims, coords = generate_dims_coords(
@@ -215,7 +216,10 @@ def make_attrs(attrs=None, library=None):
     dict
         attrs
     """
-    default_attrs = {"created_at": datetime.datetime.utcnow().isoformat()}
+    default_attrs = {
+        "created_at": datetime.datetime.utcnow().isoformat(),
+        "arviz_version": __version__,
+    }
     if library is not None:
         library_name = library.__name__
         default_attrs["inference_library"] = library_name
