@@ -3,12 +3,28 @@ from collections import OrderedDict
 from collections.abc import Sequence
 from copy import copy as ccopy, deepcopy
 from datetime import datetime
+import warnings
+
 import netCDF4 as nc
 import numpy as np
 import xarray as xr
 
 from ..rcparams import rcParams
 
+SUPPORTED_GROUPS = [
+    "posterior",
+    "sample_stats",
+    "log_likelihood",
+    "posterior_predictive",
+    "observed_data",
+    "constant_data",
+    "prior",
+    "sample_stats_prior",
+    "prior_predictive",
+    "predictions",
+    "predictions_constant_data",
+]
+
 
 class InferenceData:
     """Container for inference data storage using xarray.
@@ -60,13 +76,21 @@ class InferenceData:
 
         """
         self._groups = []
-        for key, dataset in kwargs.items():
+        key_list = [key for key in SUPPORTED_GROUPS if key in kwargs]
+        for key in kwargs:
+            if key not in SUPPORTED_GROUPS:
+                key_list.append(key)
+                warnings.warn(
+                    "{} group is not defined in the InferenceData scheme".format(key), UserWarning
+                )
+        for key in key_list:
+            dataset = kwargs[key]
             if dataset is None:
                 continue
             elif not isinstance(dataset, xr.Dataset):
                 raise ValueError(
                     "Arguments to InferenceData must be xarray Datasets "
-                    '(argument "{}" was type "{}")'.format(key, type(dataset))
+                    "(argument '{}' was type '{}')".format(key, type(dataset))
                 )
             setattr(self, key, dataset)
             self._groups.append(key)
@@ -252,6 +276,49 @@ def concat(*args, dim=None, copy=True, inplace=False, reset_dim=True):
     InferenceData
         A new InferenceData object by default.
         When `inplace==True` merge args to first arg and return `None`
+
+    Examples
+    --------
+    Use ``concat`` method to concatenate InferenceData objects. This will concatenates over
+    unique groups by default. We first create an ``InferenceData`` object:
+
+    .. ipython::
+
+        In [1]: import arviz as az
+           ...: import numpy as np
+           ...: data = {
+           ...:     "a": np.random.normal(size=(4, 100, 3)),
+           ...:     "b": np.random.normal(size=(4, 100)),
+           ...: }
+           ...: coords = {"a_dim": ["x", "y", "z"]}
+           ...: dataA = az.from_dict(data, coords=coords, dims={"a": ["a_dim"]})
+           ...: dataA
+
+    We have created an ``InferenceData`` object with default group 'posterior'. Now, we will
+    create another ``InferenceData`` object:
+
+    .. ipython::
+
+        In [1]: dataB = az.from_dict(prior=data, coords=coords, dims={"a": ["a_dim"]})
+           ...: dataB
+
+    We have created another ``InferenceData`` object with group 'prior'. Now, we will concatenate
+    these two ``InferenceData`` objects:
+
+    .. ipython::
+
+        In [1]: az.concat(dataA, dataB)
+
+    Now, we will concatenate over chain (or draw). It requires identical groups and variables.
+    Here we are concatenating two identical ``InferenceData`` objects over dimension chain:
+
+    .. ipython::
+
+        In [1]: az.concat(dataA, dataA, dim="chain")
+
+    It will create an ``InferenceData`` with the original group 'posterior'. In similar way,
+    we can also concatenate over draws.
+
     """
     # pylint: disable=undefined-loop-variable, too-many-nested-blocks
     if len(args) == 0:
@@ -312,18 +379,9 @@ def concat(*args, dim=None, copy=True, inplace=False, reset_dim=True):
                 group_data = getattr(arg0, group)
                 args_groups[group] = deepcopy(group_data) if copy else group_data
 
-        basic_order = [
-            "posterior",
-            "posterior_predictive",
-            "sample_stats",
-            "prior",
-            "prior_predictive",
-            "sample_stats_prior",
-            "observed_data",
-        ]
-        other_groups = [group for group in args_groups if group not in basic_order]
+        other_groups = [group for group in args_groups if group not in SUPPORTED_GROUPS]
 
-        for group in basic_order + other_groups:
+        for group in SUPPORTED_GROUPS + other_groups:
             if group not in args_groups:
                 continue
             if inplace:
@@ -333,9 +391,11 @@ def concat(*args, dim=None, copy=True, inplace=False, reset_dim=True):
                 inference_data_dict[group] = args_groups[group]
         if inplace:
             other_groups = [
-                group for group in arg0_groups if group not in basic_order
+                group for group in arg0_groups if group not in SUPPORTED_GROUPS
             ] + other_groups
-            sorted_groups = [group for group in basic_order + other_groups if group in arg0._groups]
+            sorted_groups = [
+                group for group in SUPPORTED_GROUPS + other_groups if group in arg0._groups
+            ]
             setattr(arg0, "_groups", sorted_groups)
     else:
         arg0 = args[0]
