@@ -2,6 +2,7 @@
 from collections import defaultdict
 from copy import deepcopy
 from glob import glob
+from typing import Optional, Union, List
 import linecache
 import os
 import logging
@@ -13,7 +14,7 @@ import xarray as xr
 
 from .. import utils
 from .inference_data import InferenceData
-from .base import requires, dict_to_dataset, generate_dims_coords
+from .base import requires, dict_to_dataset, generate_dims_coords, CoordSpec, DimSpec
 
 _log = logging.getLogger(__name__)
 
@@ -28,10 +29,15 @@ class CmdStanConverter:
         *,
         posterior=None,
         posterior_predictive=None,
+        predictions=None,
         prior=None,
         prior_predictive=None,
         observed_data=None,
         observed_data_var=None,
+        constant_data=None,
+        constant_data_var=None,
+        predictions_constant_data=None,
+        predictions_constant_data_var=None,
         log_likelihood=None,
         coords=None,
         dims=None
@@ -57,6 +63,16 @@ class CmdStanConverter:
                 )
                 len_pp = len(posterior_predictive)
                 _log.info("glob found %d files for 'posterior_predictive':\n%s", len_pp, msg)
+        if isinstance(predictions, str):
+            predictions_glob = glob(predictions)
+            if len(predictions_glob) > 1:
+                predictions = sorted(predictions_glob)
+                msg = "\n".join(
+                    "{}: {}".format(i, os.path.normpath(path))
+                    for i, path in enumerate(predictions, 1)
+                )
+                len_p = len(predictions)
+                _log.info("glob found %d files for 'predictions':\n%s", len_p, msg)
         if isinstance(prior, str):
             prior_glob = glob(prior)
             if len(prior_glob) > 1:
@@ -76,14 +92,26 @@ class CmdStanConverter:
                 )
                 len_pp = len(prior_predictive)
                 _log.info("glob found %d files for 'prior_predictive':\n%s", len_pp, msg)
+        if isinstance(log_likelihood, str):
+            log_likelihood_glob = glob(log_likelihood)
+            if len(log_likelihood_glob) > 1:
+                log_likelihood = sorted(log_likelihood_glob)
+                msg = "\n".join(
+                    "{}: {}".format(i, os.path.normpath(path))
+                    for i, path in enumerate(log_likelihood, 1)
+                )
+                len_ll = len(log_likelihood)
+                _log.info("glob found %d files for 'log_likelihood':\n%s", len_ll, msg)
         self.posterior_predictive = posterior_predictive
+        self.predictions = predictions
         self.prior_ = prior
         self.prior_predictive = prior_predictive
         self.observed_data = observed_data
         self.observed_data_var = observed_data_var
-        if isinstance(log_likelihood, (list, tuple)):
-            if len(log_likelihood) == 1:
-                log_likelihood = log_likelihood[0]
+        self.constant_data = constant_data
+        self.constant_data_var = constant_data_var
+        self.predictions_constant_data = predictions_constant_data
+        self.predictions_constant_data_var = predictions_constant_data_var
         self.log_likelihood = log_likelihood
         self.coords = coords if coords is not None else {}
         self.dims = dims if dims is not None else {}
@@ -92,7 +120,7 @@ class CmdStanConverter:
         self.prior = None
         self.sample_stats_prior = None
 
-        # populate posterior and sample_Stats
+        # populate posterior and sample_stats
         self._parse_posterior()
         self._parse_prior()
 
@@ -145,7 +173,7 @@ class CmdStanConverter:
         """Extract posterior samples from output csv."""
         columns = self.posterior[0].columns
 
-        # filter posterior_predictive and log_likelihood
+        # filter posterior_predictive, predictions and log_likelihood
         posterior_predictive = self.posterior_predictive
         if posterior_predictive is None or (
             isinstance(posterior_predictive, str) and posterior_predictive.lower().endswith(".csv")
@@ -162,13 +190,31 @@ class CmdStanConverter:
                 if any(item == col.split(".")[0] for item in posterior_predictive)
             ]
 
-        log_likelihood = self.log_likelihood
-        if log_likelihood is None:
-            log_likelihood = []
+        predictions = self.predictions
+        if predictions is None or (
+            isinstance(predictions, str) and predictions.lower().endswith(".csv")
+        ):
+            predictions = []
+        elif isinstance(predictions, str):
+            predictions = [col for col in columns if predictions == col.split(".")[0]]
         else:
-            log_likelihood = [col for col in columns if log_likelihood == col.split(".")[0]]
+            predictions = [
+                col for col in columns if any(item == col.split(".")[0] for item in predictions)
+            ]
 
-        invalid_cols = posterior_predictive + log_likelihood
+        log_likelihood = self.log_likelihood
+        if log_likelihood is None or (
+            isinstance(log_likelihood, str) and log_likelihood.lower().endswith(".csv")
+        ):
+            log_likelihood = []
+        elif isinstance(log_likelihood, str):
+            log_likelihood = [col for col in columns if log_likelihood == col.split(".")[0]]
+        else:
+            log_likelihood = [
+                col for col in columns if any(item == col.split(".")[0] for item in log_likelihood)
+            ]
+
+        invalid_cols = posterior_predictive + predictions + log_likelihood
         valid_cols = [col for col in columns if col not in invalid_cols]
         data = _unpack_dataframes([item[valid_cols] for item in self.posterior])
         return dict_to_dataset(data, coords=self.coords, dims=self.dims)
@@ -179,43 +225,7 @@ class CmdStanConverter:
         """Extract sample_stats from fit."""
         dtypes = {"divergent__": bool, "n_leapfrog__": np.int64, "treedepth__": np.int64}
 
-        # copy dims and coords
-        dims = deepcopy(self.dims) if self.dims is not None else {}
-        coords = deepcopy(self.coords) if self.coords is not None else {}
-
         sampler_params = self.sample_stats
-        log_likelihood = self.log_likelihood
-        if isinstance(log_likelihood, str):
-            log_likelihood_cols = [
-                col for col in self.posterior[0].columns if log_likelihood == col.split(".")[0]
-            ]
-            log_likelihood_vals = [item[log_likelihood_cols] for item in self.posterior]
-
-            # Add log_likelihood to sampler_params
-            for i, _ in enumerate(sampler_params):
-                # slice log_likelihood to keep dimensions
-                for col in log_likelihood_cols:
-                    col_ll = col.replace(log_likelihood, "log_likelihood")
-                    sampler_params[i][col_ll] = log_likelihood_vals[i][col]
-
-            # change dims and coords for log_likelihood if defined
-            if log_likelihood in dims:
-                dims["log_likelihood"] = dims.pop(log_likelihood)
-
-            log_likelihood_dims = np.array(
-                [list(map(int, col.split(".")[1:])) for col in log_likelihood_cols]
-            )
-            max_dims = log_likelihood_dims.max(0)
-            max_dims = max_dims if hasattr(max_dims, "__iter__") else (max_dims,)
-            default_dim_names, _ = generate_dims_coords(shape=max_dims, var_name=log_likelihood)
-            log_likelihood_dim_names, _ = generate_dims_coords(
-                shape=max_dims, var_name="log_likelihood"
-            )
-            for default_dim_name, log_likelihood_dim_name in zip(
-                default_dim_names, log_likelihood_dim_names
-            ):
-                if default_dim_name in coords:
-                    coords[log_likelihood_dim_name] = coords.pop(default_dim_name)
 
         for j, s_params in enumerate(sampler_params):
             rename_dict = {}
@@ -227,7 +237,7 @@ class CmdStanConverter:
                 sampler_params[j][key] = s_params[key].astype(dtypes.get(key_))
             sampler_params[j] = sampler_params[j].rename(columns=rename_dict)
         data = _unpack_dataframes(sampler_params)
-        return dict_to_dataset(data, coords=coords, dims=dims)
+        return dict_to_dataset(data, coords=self.coords, dims=self.dims)
 
     @requires("posterior")
     @requires("posterior_predictive")
@@ -258,6 +268,32 @@ class CmdStanConverter:
             data = _unpack_dataframes([item[posterior_predictive_cols] for item in self.posterior])
         return dict_to_dataset(data, coords=self.coords, dims=self.dims)
 
+    @requires("posterior")
+    @requires("predictions")
+    def predictions_to_xarray(self):
+        """Convert out of sample predictions samples to xarray."""
+        predictions = self.predictions
+        columns = self.posterior[0].columns
+        if (isinstance(predictions, (tuple, list)) and predictions[0].endswith(".csv")) or (
+            isinstance(predictions, str) and predictions.endswith(".csv")
+        ):
+            if isinstance(predictions, str):
+                predictions = [predictions]
+            chain_data = []
+            for path in predictions:
+                parsed_output = _read_output(path)
+                for sample, *_ in parsed_output:
+                    chain_data.append(sample)
+            data = _unpack_dataframes(chain_data)
+        else:
+            if isinstance(predictions, str):
+                predictions = [predictions]
+            predictions_cols = [
+                col for col in columns if any(item == col.split(".")[0] for item in predictions)
+            ]
+            data = _unpack_dataframes([item[predictions_cols] for item in self.posterior])
+        return dict_to_dataset(data, coords=self.coords, dims=self.dims)
+
     @requires("prior")
     def prior_to_xarray(self):
         """Convert prior samples to xarray."""
@@ -352,6 +388,70 @@ class CmdStanConverter:
             observed_data[key] = xr.DataArray(vals, dims=val_dims, coords=coords)
         return xr.Dataset(data_vars=observed_data)
 
+    @requires("constant_data")
+    def constant_data_to_xarray(self):
+        """Convert constant data to xarray."""
+        constant_data_raw = _read_data(self.constant_data)
+        variables = self.constant_data_var
+        if isinstance(variables, str):
+            variables = [variables]
+        constant_data = {}
+        for key, vals in constant_data_raw.items():
+            if variables is not None and key not in variables:
+                continue
+            vals = utils.one_de(vals)
+            val_dims = self.dims.get(key)
+            val_dims, coords = generate_dims_coords(
+                vals.shape, key, dims=val_dims, coords=self.coords
+            )
+            constant_data[key] = xr.DataArray(vals, dims=val_dims, coords=coords)
+        return xr.Dataset(data_vars=constant_data)
+
+    @requires("predictions_constant_data")
+    def predictions_constant_data_to_xarray(self):
+        """Convert predictions constant data to xarray."""
+        predictions_constant_data_raw = _read_data(self.predictions_constant_data)
+        variables = self.predictions_constant_data_var
+        if isinstance(variables, str):
+            variables = [variables]
+        predictions_constant_data = {}
+        for key, vals in predictions_constant_data_raw.items():
+            if variables is not None and key not in variables:
+                continue
+            vals = utils.one_de(vals)
+            val_dims = self.dims.get(key)
+            val_dims, coords = generate_dims_coords(
+                vals.shape, key, dims=val_dims, coords=self.coords
+            )
+            predictions_constant_data[key] = xr.DataArray(vals, dims=val_dims, coords=coords)
+        return xr.Dataset(data_vars=predictions_constant_data)
+
+    @requires("posterior")
+    @requires("log_likelihood")
+    def log_likelihood_to_xarray(self):
+        """Convert elementwise log_likelihood samples to xarray."""
+        log_likelihood = self.log_likelihood
+        columns = self.posterior[0].columns
+        if (isinstance(log_likelihood, (tuple, list)) and log_likelihood[0].endswith(".csv")) or (
+            isinstance(log_likelihood, str) and log_likelihood.endswith(".csv")
+        ):
+            if isinstance(log_likelihood, str):
+                log_likelihood = [log_likelihood]
+            chain_data = []
+            for path in log_likelihood:
+                parsed_output = _read_output(path)
+                for sample, *_ in parsed_output:
+                    chain_data.append(sample)
+            data = _unpack_dataframes(chain_data)
+        else:
+            if isinstance(log_likelihood, str):
+                log_likelihood = [log_likelihood]
+            log_likelihood_cols = [
+                col for col in columns if any(item == col.split(".")[0] for item in log_likelihood)
+            ]
+            data = _unpack_dataframes([item[log_likelihood_cols] for item in self.posterior])
+        return dict_to_dataset(data, coords=self.coords, dims=self.dims)
+
     def to_inference_data(self):
         """Convert all available data to an InferenceData object.
 
@@ -363,11 +463,15 @@ class CmdStanConverter:
             **{
                 "posterior": self.posterior_to_xarray(),
                 "sample_stats": self.sample_stats_to_xarray(),
+                "log_likelihood": self.log_likelihood_to_xarray(),
                 "posterior_predictive": self.posterior_predictive_to_xarray(),
                 "prior": self.prior_to_xarray(),
                 "sample_stats_prior": self.sample_stats_prior_to_xarray(),
                 "prior_predictive": self.prior_predictive_to_xarray(),
                 "observed_data": self.observed_data_to_xarray(),
+                "constant_data": self.constant_data_to_xarray(),
+                "predictions": self.predictions_to_xarray(),
+                "predictions_constant_data": self.predictions_constant_data_to_xarray(),
             }
         )
 
@@ -669,48 +773,66 @@ def _unpack_dataframes(dfs):
 
 
 def from_cmdstan(
-    posterior=None,
+    posterior: Optional[Union[str, List[str]]] = None,
     *,
-    posterior_predictive=None,
-    prior=None,
-    prior_predictive=None,
-    observed_data=None,
-    observed_data_var=None,
-    log_likelihood=None,
-    coords=None,
-    dims=None
-):
+    posterior_predictive: Optional[Union[str, List[str]]] = None,
+    predictions: Optional[Union[str, List[str]]] = None,
+    prior: Optional[Union[str, List[str]]] = None,
+    prior_predictive: Optional[Union[str, List[str]]] = None,
+    observed_data: Optional[str] = None,
+    observed_data_var: Optional[Union[str, List[str]]] = None,
+    constant_data: Optional[str] = None,
+    constant_data_var: Optional[Union[str, List[str]]] = None,
+    predictions_constant_data: Optional[str] = None,
+    predictions_constant_data_var: Optional[Union[str, List[str]]] = None,
+    log_likelihood: Optional[Union[str, List[str]]] = None,
+    coords: Optional[CoordSpec] = None,
+    dims: Optional[DimSpec] = None
+) -> InferenceData:
     """Convert CmdStan data into an InferenceData object.
 
     Parameters
     ----------
-    posterior : List[str]
+    posterior : str or list of str, optional
         List of paths to output.csv files.
         CSV file can be stacked csv containing all the chains
 
             cat output*.csv > combined_output.csv
 
-    posterior_predictive : str, List[Str]
+    posterior_predictive : str or list of str, optional
         Posterior predictive samples for the fit. If endswith ".csv" assumes file.
-    prior : List[str]
+    predictions : str or list of str, optional
+        Out of sample predictions samples for the fit. If endswith ".csv" assumes file.
+    prior : str or list of str, optional
         List of paths to output.csv files
         CSV file can be stacked csv containing all the chains.
 
             cat output*.csv > combined_output.csv
 
-    prior_predictive : str, List[Str]
+    prior_predictive : str or list of str, optional
         Prior predictive samples for the fit. If endswith ".csv" assumes file.
-    observed_data : str
+    observed_data : str, optional
         Observed data used in the sampling. Path to data file in Rdump format.
-    observed_data_var : str, List[str]
+    observed_data_var : str or list of str, optional
         Variable(s) used for slicing observed_data. If not defined, all
         data variables are imported.
-    log_likelihood : str
+    constant_data : str, optional
+        Constant data used in the sampling. Path to data file in Rdump format.
+    constant_data_var : str or list of str, optional
+        Variable(s) used for slicing constant_data. If not defined, all
+        data variables are imported.
+    predictions_constant_data : str, optional
+        Constant data for predictions used in the sampling.
+        Path to data file in Rdump format.
+    predictions_constant_data_var : str or list of str, optional
+        Variable(s) used for slicing predictions_constant_data.
+        If not defined, all data variables are imported.
+    log_likelihood : str or list of str, optional
         Pointwise log_likelihood for the data.
-    coords : dict[str, iterable]
+    coords : dict of {str: array_like}, optional
         A dictionary containing the values that are used as index. The key
         is the name of the dimension, the values are the index values.
-    dims : dict[str, List(str)]
+    dims : dict of {str: list of str, optional
         A mapping from variables to a list of coordinate names for the variable.
 
     Returns
@@ -720,10 +842,15 @@ def from_cmdstan(
     return CmdStanConverter(
         posterior=posterior,
         posterior_predictive=posterior_predictive,
+        predictions=predictions,
         prior=prior,
         prior_predictive=prior_predictive,
         observed_data=observed_data,
         observed_data_var=observed_data_var,
+        constant_data=constant_data,
+        constant_data_var=constant_data_var,
+        predictions_constant_data=predictions_constant_data,
+        predictions_constant_data_var=predictions_constant_data_var,
         log_likelihood=log_likelihood,
         coords=coords,
         dims=dims,
