@@ -7,6 +7,7 @@ from .base import requires, dict_to_dataset, generate_dims_coords, make_attrs
 from .. import utils
 
 
+# pylint: disable=too-many-instance-attributes
 class DictConverter:
     """Encapsulate Dictionary specific logic."""
 
@@ -16,20 +17,24 @@ class DictConverter:
         posterior=None,
         posterior_predictive=None,
         sample_stats=None,
+        log_likelihood=None,
         prior=None,
         prior_predictive=None,
         sample_stats_prior=None,
         observed_data=None,
+        constant_data=None,
         coords=None,
         dims=None
     ):
         self.posterior = posterior
         self.posterior_predictive = posterior_predictive
         self.sample_stats = sample_stats
+        self.log_likelihood = log_likelihood
         self.prior = prior
         self.prior_predictive = prior_predictive
         self.sample_stats_prior = sample_stats_prior
         self.observed_data = observed_data
+        self.constant_data = constant_data
         self.coords = coords
         self.dims = dims
 
@@ -42,9 +47,9 @@ class DictConverter:
 
         if "log_likelihood" in data:
             warnings.warn(
-                "log_likelihood found in posterior."
-                " For stats functions log_likelihood needs to be in sample_stats.",
-                SyntaxWarning,
+                "log_likelihood variable found in posterior group."
+                " For stats functions log likelihood data needs to be in log_likelihood group.",
+                UserWarning,
             )
 
         return dict_to_dataset(data, library=None, coords=self.coords, dims=self.dims)
@@ -56,6 +61,23 @@ class DictConverter:
         if not isinstance(data, dict):
             raise TypeError("DictConverter.sample_stats is not a dictionary")
 
+        if "log_likelihood" in data:
+            warnings.warn(
+                "log_likelihood variable found in sample_stats."
+                " Storing log_likelihood data in sample_stats group will be deprecated in "
+                "favour of storing them in the log_likelihood group.",
+                PendingDeprecationWarning,
+            )
+
+        return dict_to_dataset(data, library=None, coords=self.coords, dims=self.dims)
+
+    @requires("log_likelihood")
+    def log_likelihood_to_xarray(self):
+        """Convert log_likelihood samples to xarray."""
+        data = self.log_likelihood
+        if not isinstance(data, dict):
+            raise TypeError("DictConverter.log_likelihood is not a dictionary")
+
         return dict_to_dataset(data, library=None, coords=self.coords, dims=self.dims)
 
     @requires("posterior_predictive")
@@ -114,6 +136,26 @@ class DictConverter:
             observed_data[key] = xr.DataArray(vals, dims=val_dims, coords=coords)
         return xr.Dataset(data_vars=observed_data, attrs=make_attrs(library=None))
 
+    @requires("constant_data")
+    def constant_data_to_xarray(self):
+        """Convert constant_data to xarray."""
+        data = self.constant_data
+        if not isinstance(data, dict):
+            raise TypeError("DictConverter.constant_data is not a dictionary")
+        if self.dims is None:
+            dims = {}
+        else:
+            dims = self.dims
+        constant_data = dict()
+        for key, vals in data.items():
+            vals = utils.one_de(vals)
+            val_dims = dims.get(key)
+            val_dims, coords = generate_dims_coords(
+                vals.shape, key, dims=val_dims, coords=self.coords
+            )
+            constant_data[key] = xr.DataArray(vals, dims=val_dims, coords=coords)
+        return xr.Dataset(data_vars=constant_data, attrs=make_attrs(library=None))
+
     def to_inference_data(self):
         """Convert all available data to an InferenceData object.
 
@@ -124,25 +166,29 @@ class DictConverter:
             **{
                 "posterior": self.posterior_to_xarray(),
                 "sample_stats": self.sample_stats_to_xarray(),
+                "log_likelihood": self.log_likelihood_to_xarray(),
                 "posterior_predictive": self.posterior_predictive_to_xarray(),
                 "prior": self.prior_to_xarray(),
                 "sample_stats_prior": self.sample_stats_prior_to_xarray(),
                 "prior_predictive": self.prior_predictive_to_xarray(),
                 "observed_data": self.observed_data_to_xarray(),
+                "constant_data": self.constant_data_to_xarray(),
             }
         )
 
 
-# pylint disable=too-many-instance-attributes
+# pylint: disable=too-many-instance-attributes
 def from_dict(
     posterior=None,
     *,
     posterior_predictive=None,
     sample_stats=None,
+    log_likelihood=None,
     prior=None,
     prior_predictive=None,
     sample_stats_prior=None,
     observed_data=None,
+    constant_data=None,
     coords=None,
     dims=None
 ):
@@ -153,10 +199,12 @@ def from_dict(
     posterior : dict
     posterior_predictive : dict
     sample_stats : dict
-        "log_likelihood" variable for stats needs to be here.
+    log_likelihood : dict
+        For stats functions, log likelihood data should be stored here.
     prior : dict
     prior_predictive : dict
     observed_data : dict
+    constant_data : dict
     coords : dict[str, iterable]
         A dictionary containing the values that are used as index. The key
         is the name of the dimension, the values are the index values.
@@ -171,10 +219,12 @@ def from_dict(
         posterior=posterior,
         posterior_predictive=posterior_predictive,
         sample_stats=sample_stats,
+        log_likelihood=log_likelihood,
         prior=prior,
         prior_predictive=prior_predictive,
         sample_stats_prior=sample_stats_prior,
         observed_data=observed_data,
+        constant_data=constant_data,
         coords=coords,
         dims=dims,
     ).to_inference_data()
