@@ -4,16 +4,15 @@ from collections import defaultdict, OrderedDict
 from itertools import cycle, tee
 
 import bokeh.plotting as bkp
-import matplotlib.pyplot as plt
-import numpy as np
-from bokeh.layouts import gridplot
-from bokeh.models import Band, ColumnDataSource
+from bokeh.models import Band, ColumnDataSource, DataRange1d
 from bokeh.models.annotations import Title
 from bokeh.models.tickers import FixedTicker
+import matplotlib.pyplot as plt
+import numpy as np
 
-from . import backend_kwarg_defaults, backend_show
-from ...kdeplot import _fast_kde
-from ...plot_utils import _scale_fig_size, xarray_var_iter, make_label, get_bins
+from . import backend_kwarg_defaults
+from .. import show_layout
+from ...plot_utils import _scale_fig_size, xarray_var_iter, make_label, get_bins, _fast_kde
 from ....rcparams import rcParams
 from ....stats import hpd
 from ....stats.diagnostics import _ess, _rhat
@@ -47,9 +46,11 @@ def plot_forest(
     ridgeplot_overlap,
     ridgeplot_alpha,
     ridgeplot_kind,
+    ridgeplot_quantiles,
     textsize,
     ess,
     r_hat,
+    backend_config,
     backend_kwargs,
     show,
 ):
@@ -59,7 +60,10 @@ def plot_forest(
     )
 
     if figsize is None:
-        figsize = (min(12, sum(width_ratios) * 2), plot_handler.fig_height())
+        if kind == "ridgeplot":
+            figsize = (min(14, sum(width_ratios) * 3), plot_handler.fig_height() * 3)
+        else:
+            figsize = (min(12, sum(width_ratios) * 2), plot_handler.fig_height())
 
     (figsize, _, _, _, auto_linewidth, auto_markersize) = _scale_fig_size(figsize, textsize, 1.1, 1)
 
@@ -69,36 +73,40 @@ def plot_forest(
     if markersize is None:
         markersize = auto_markersize
 
+    if backend_config is None:
+        backend_config = {}
+
+    backend_config = {
+        **backend_kwarg_defaults(
+            ("bounds_x_range", "plot.bokeh.bounds_x_range"),
+            ("bounds_y_range", "plot.bokeh.bounds_y_range"),
+        ),
+        **backend_config,
+    }
+
     if backend_kwargs is None:
         backend_kwargs = {}
 
     backend_kwargs = {
-        **backend_kwarg_defaults(
-            ("tools", "plot.bokeh.tools"),
-            ("output_backend", "plot.bokeh.output_backend"),
-            ("dpi", "plot.bokeh.figure.dpi"),
-        ),
+        **backend_kwarg_defaults(("dpi", "plot.bokeh.figure.dpi"),),
         **backend_kwargs,
     }
     dpi = backend_kwargs.pop("dpi")
 
     if ax is None:
         axes = []
+
         for i, width_r in zip(range(ncols), width_ratios):
+            backend_kwargs_i = backend_kwargs.copy()
+            backend_kwargs_i.setdefault("width", int(figsize[0] * dpi))
+            backend_kwargs_i.setdefault(
+                "height", int(figsize[1] * (width_r / sum(width_ratios)) * dpi * 1.25)
+            )
             if i == 0:
-                ax = bkp.figure(
-                    height=int(figsize[0]) * dpi,
-                    width=int(figsize[1] * (width_r / sum(width_ratios)) * dpi * 1.25),
-                    **backend_kwargs,
-                )
+                ax = bkp.figure(**backend_kwargs,)
                 _y_range = ax.y_range
             else:
-                ax = bkp.figure(
-                    height=figsize[0] * dpi,
-                    width=int(figsize[1] * (width_r / sum(width_ratios)) * dpi * 1.25),
-                    y_range=_y_range,
-                    **backend_kwargs,
-                )
+                ax = bkp.figure(y_range=_y_range, **backend_kwargs,)
             axes.append(ax)
     else:
         axes = ax
@@ -111,7 +119,12 @@ def plot_forest(
         )
     elif kind == "ridgeplot":
         plot_handler.ridgeplot(
-            ridgeplot_overlap, linewidth, ridgeplot_alpha, ridgeplot_kind, axes[0, 0]
+            ridgeplot_overlap,
+            linewidth,
+            ridgeplot_alpha,
+            ridgeplot_kind,
+            ridgeplot_quantiles,
+            axes[0, 0],
         )
     else:
         raise TypeError(
@@ -139,6 +152,8 @@ def plot_forest(
             ax_.yaxis.visible = False
 
         ax_.outline_line_color = None
+        ax_.x_range = DataRange1d(bounds=backend_config["bounds_x_range"], min_interval=1)
+        ax_.y_range = DataRange1d(bounds=backend_config["bounds_y_range"], min_interval=2)
 
     labels, ticks = plot_handler.labels_and_ticks()
 
@@ -157,9 +172,7 @@ def plot_forest(
     ].group_offset
     axes[0, 0].y_range._property_values["end"] = y_max  # pylint: disable=protected-access
 
-    if backend_show(show):
-        grid = gridplot(axes.tolist(), toolbar_location="above")
-        bkp.show(grid)
+    show_layout(axes, show)
 
     return axes
 
@@ -257,7 +270,7 @@ class PlotHandler:
         )
         return ax
 
-    def ridgeplot(self, mult, linewidth, alpha, ridgeplot_kind, ax):
+    def ridgeplot(self, mult, linewidth, alpha, ridgeplot_kind, ridgeplot_quantiles, ax):
         """Draw ridgeplot for each plotter.
 
         Parameters
@@ -277,7 +290,7 @@ class PlotHandler:
         if alpha is None:
             alpha = 1.0
         for plotter in list(self.plotters.values())[::-1]:
-            for x, y_min, y_max, color in list(plotter.ridgeplot(mult, ridgeplot_kind))[::-1]:
+            for x, y_min, y_max, y_q, color in list(plotter.ridgeplot(mult, ridgeplot_kind))[::-1]:
                 if alpha == 0:
                     border = color
                     facecolor = None
@@ -293,16 +306,43 @@ class PlotHandler:
                         fill_color=facecolor,
                     )
                 else:
-                    patch = ax.patch(
-                        np.concatenate([x, x[::-1]]),
-                        np.concatenate([y_min, y_max[::-1]]),
-                        fill_color=color,
-                        fill_alpha=alpha,
-                        line_dash="solid",
-                        line_width=linewidth,
-                        line_color=border,
-                    )
-                    patch.level = "overlay"
+                    if ridgeplot_quantiles is None:
+                        patch = ax.patch(
+                            np.concatenate([x, x[::-1]]),
+                            np.concatenate([y_min, y_max[::-1]]),
+                            fill_color=color,
+                            fill_alpha=alpha,
+                            line_dash="solid",
+                            line_width=linewidth,
+                            line_color=border,
+                        )
+                        patch.level = "overlay"
+                    else:
+                        quantiles = sorted(np.clip(ridgeplot_quantiles, 0, 1))
+                        if quantiles[0] != 0:
+                            quantiles = [0] + quantiles
+                        if quantiles[-1] != 1:
+                            quantiles = quantiles + [1]
+
+                        for quant_0, quant_1 in zip(quantiles[:-1], quantiles[1:]):
+                            idx = (y_q > quant_0) & (y_q < quant_1)
+                            if idx.sum():
+                                patch_x = np.concatenate(
+                                    (x[idx], [x[idx][-1]], x[idx][::-1], [x[idx][0]])
+                                )
+                                patch_y = np.concatenate(
+                                    (
+                                        y_min[idx],
+                                        [y_min[idx][-1]],
+                                        y_max[idx][::-1],
+                                        [y_max[idx][0]],
+                                    )
+                                )
+                                patch = ax.patch(
+                                    patch_x, patch_y, fill_color=color, fill_alpha=alpha,
+                                )
+                            patch.level = "overlay"
+
         return ax
 
     def forestplot(self, credible_interval, quartiles, linewidth, markersize, ax, rope):
@@ -513,7 +553,7 @@ class VarHandler:
 
     def ridgeplot(self, mult, ridgeplot_kind):
         """Get data for each ridgeplot for the variable."""
-        xvals, yvals, pdfs, colors = [], [], [], []
+        xvals, yvals, pdfs, pdfs_q, colors = [], [], [], [], []
         for y, *_, values, color in self.iterator():
             yvals.append(y)
             colors.append(color)
@@ -531,15 +571,17 @@ class VarHandler:
                 x = x[:-1]
             elif kind == "density":
                 density, lower, upper = _fast_kde(values)
+                density_q = density.cumsum() / density.sum()
                 x = np.linspace(lower, upper, len(density))
 
             xvals.append(x)
             pdfs.append(density)
+            pdfs_q.append((density_q))
 
         scaling = max(np.max(j) for j in pdfs)
-        for y, x, pdf, color in zip(yvals, xvals, pdfs, colors):
+        for y, x, pdf, pdf_q, color in zip(yvals, xvals, pdfs, pdfs_q, colors):
             y = y * np.ones_like(x)
-            yield x, y, mult * pdf / scaling + y, color
+            yield x, y, mult * pdf / scaling + y, pdf_q, color
 
     def ess(self):
         """Get effective n data for the variable."""
