@@ -1,14 +1,13 @@
 """Bokeh pairplot."""
-
 import warnings
 from uuid import uuid4
 
 import bokeh.plotting as bkp
-import numpy as np
-from bokeh.layouts import gridplot
 from bokeh.models import ColumnDataSource, CDSView, GroupFilter
+import numpy as np
 
-from . import backend_kwarg_defaults, backend_show
+from . import backend_kwarg_defaults
+from .. import show_layout
 from ...kdeplot import plot_kde
 from ...plot_utils import _scale_fig_size
 from ....rcparams import rcParams
@@ -16,7 +15,7 @@ from ....rcparams import rcParams
 
 def plot_pair(
     ax,
-    _posterior,
+    infdata_group,
     numvars,
     figsize,
     textsize,
@@ -35,18 +34,21 @@ def plot_pair(
         backend_kwargs = {}
 
     backend_kwargs = {
-        **backend_kwarg_defaults(
-            ("tools", "plot.bokeh.tools"),
-            ("output_backend", "plot.bokeh.output_backend"),
-            ("dpi", "plot.bokeh.figure.dpi"),
-        ),
+        **backend_kwarg_defaults(("dpi", "plot.bokeh.figure.dpi"),),
         **backend_kwargs,
     }
     dpi = backend_kwargs.pop("dpi")
     if numvars == 2:
+        if kind == "scatter":
+            tooltips = [
+                (flat_var_names[1], "@{{{}}}".format(flat_var_names[1])),
+                (flat_var_names[0], "@{{{}}}".format(flat_var_names[0])),
+            ]
+            backend_kwargs.setdefault("tooltips", tooltips)
+
         (figsize, _, _, _, _, _) = _scale_fig_size(figsize, textsize, numvars - 1, numvars - 1)
 
-        source_dict = dict(zip(flat_var_names, [list(post) for post in _posterior]))
+        source_dict = dict(zip(flat_var_names, [list(post) for post in infdata_group]))
 
         if divergences:
             divergenve_name = "divergences_{}".format(str(uuid4()))
@@ -65,9 +67,9 @@ def plot_pair(
             )
 
         if ax is None:
-            ax = bkp.figure(
-                width=int(figsize[0] * dpi), height=int(figsize[1] * dpi), **backend_kwargs
-            )
+            backend_kwargs["width"] = int(figsize[0] * dpi)
+            backend_kwargs["height"] = int(figsize[1] * dpi)
+            ax = bkp.figure(**backend_kwargs)
 
         if kind == "scatter":
             if divergences:
@@ -82,8 +84,8 @@ def plot_pair(
                 ax.circle(flat_var_names[0], flat_var_names[1], source=source)
         elif kind == "kde":
             plot_kde(
-                _posterior[0],
-                _posterior[1],
+                infdata_group[0],
+                infdata_group[1],
                 contour=contour,
                 fill_last=fill_last,
                 ax=ax,
@@ -92,7 +94,7 @@ def plot_pair(
                 show=False,
             )
         else:
-            ax.hexbin(_posterior[0], _posterior[1], size=0.5)
+            ax.hexbin(infdata_group[0], infdata_group[1], size=0.5)
             ax.grid.visible = False
 
         if divergences:
@@ -112,8 +114,7 @@ def plot_pair(
         ax.xaxis.axis_label = flat_var_names[0]
         ax.yaxis.axis_label = flat_var_names[1]
 
-        if backend_show(show):
-            bkp.show(ax)
+        show_layout(ax, show)
 
     else:
         max_plots = (
@@ -125,35 +126,18 @@ def plot_pair(
                 "rcParams['plot.max_subplots'] ({max_plots}) is smaller than the number "
                 "of resulting pair plots with these variables, generating only a "
                 "{side}x{side} grid".format(max_plots=max_plots, side=vars_to_plot),
-                SyntaxWarning,
+                UserWarning,
             )
             numvars = vars_to_plot
 
         (figsize, _, _, _, _, _) = _scale_fig_size(figsize, textsize, numvars - 2, numvars - 2)
 
-        if ax is None:
-            ax = []
-            for row in range(numvars - 1):
-                row_ax = []
-                for col in range(numvars - 1):
-                    if row < col:
-                        row_ax.append(None)
-                    else:
-                        ax_ = bkp.figure(
-                            width=int(figsize[0] / (numvars - 1) * dpi),
-                            height=int(figsize[1] / (numvars - 1) * dpi),
-                            **backend_kwargs
-                        )
-                        row_ax.append(ax_)
-                ax.append(row_ax)
-            ax = np.array(ax)
-
         tmp_flat_var_names = None
         if len(flat_var_names) == len(list(set(flat_var_names))):
-            source_dict = dict(zip(flat_var_names, [list(post) for post in _posterior]))
+            source_dict = dict(zip(flat_var_names, [list(post) for post in infdata_group]))
         else:
             tmp_flat_var_names = ["{}__{}".format(name, str(uuid4())) for name in flat_var_names]
-            source_dict = dict(zip(tmp_flat_var_names, [list(post) for post in _posterior]))
+            source_dict = dict(zip(tmp_flat_var_names, [list(post) for post in infdata_group]))
         if divergences:
             divergenve_name = "divergences_{}".format(str(uuid4()))
             source_dict[divergenve_name] = (
@@ -170,6 +154,40 @@ def plot_pair(
                 source=source, filters=[GroupFilter(column_name=divergenve_name, group="1")]
             )
 
+        if ax is None:
+            ax = []
+            backend_kwargs.setdefault("width", int(figsize[0] / (numvars - 1) * dpi))
+            backend_kwargs.setdefault("height", int(figsize[1] / (numvars - 1) * dpi))
+            for row in range(numvars - 1):
+                row_ax = []
+                var2 = (
+                    flat_var_names[row + 1]
+                    if tmp_flat_var_names is None
+                    else tmp_flat_var_names[row + 1]
+                )
+                for col in range(numvars - 1):
+                    if row < col:
+                        row_ax.append(None)
+                        continue
+
+                    var1 = (
+                        flat_var_names[col]
+                        if tmp_flat_var_names is None
+                        else tmp_flat_var_names[col]
+                    )
+                    backend_kwargs_copy = backend_kwargs.copy()
+                    if kind == "scatter":
+                        tooltips = [
+                            (var2, "@{{{}}}".format(var2)),
+                            (var1, "@{{{}}}".format(var1)),
+                        ]
+                        backend_kwargs_copy.setdefault("tooltips", tooltips)
+
+                    ax_ = bkp.figure(**backend_kwargs_copy)
+                    row_ax.append(ax_)
+                ax.append(row_ax)
+            ax = np.array(ax)
+
         for i in range(0, numvars - 1):
             var1 = flat_var_names[i] if tmp_flat_var_names is None else tmp_flat_var_names[i]
 
@@ -190,8 +208,8 @@ def plot_pair(
                         ax[j, i].circle(var1, var2, source=source)
 
                 elif kind == "kde":
-                    var1_kde = _posterior[i]
-                    var2_kde = _posterior[j + 1]
+                    var1_kde = infdata_group[i]
+                    var2_kde = infdata_group[j + 1]
                     plot_kde(
                         var1_kde,
                         var2_kde,
@@ -205,8 +223,8 @@ def plot_pair(
                     )
 
                 else:
-                    var1_hexbin = _posterior[i]
-                    var2_hexbin = _posterior[j + 1]
+                    var1_hexbin = infdata_group[i]
+                    var2_hexbin = infdata_group[j + 1]
                     ax[j, i].grid.visible = False
                     ax[j, i].hexbin(var1_hexbin, var2_hexbin, size=0.5)
 
@@ -225,8 +243,6 @@ def plot_pair(
                 ax[j, i].xaxis.axis_label = flat_var_names[i]
                 ax[j, i].yaxis.axis_label = flat_var_names[j + 1]
 
-        if backend_show(show):
-            grid = gridplot(ax.tolist(), toolbar_location="above")
-            bkp.show(grid)
+        show_layout(ax, show)
 
     return ax
