@@ -1,10 +1,9 @@
 """Bokeh Violinplot."""
-import bokeh.plotting as bkp
-import numpy as np
-from bokeh.layouts import gridplot
 from bokeh.models.annotations import Title
+import numpy as np
 
-from . import backend_kwarg_defaults, backend_show
+from . import backend_kwarg_defaults
+from .. import show_layout
 from ...kdeplot import _fast_kde
 from ...plot_utils import get_bins, make_label, _create_axes_grid
 from ....stats import hpd
@@ -17,9 +16,12 @@ def plot_violin(
     figsize,
     rows,
     cols,
+    sharex,
     sharey,
-    kwargs_shade,
+    shade_kwargs,
     shade,
+    rug,
+    rug_kwargs,
     bw,
     credible_interval,
     linewidth,
@@ -32,7 +34,7 @@ def plot_violin(
         backend_kwargs = {}
 
     backend_kwargs = {
-        **backend_kwarg_defaults(),
+        **backend_kwarg_defaults(("dpi", "plot.bokeh.figure.dpi"),),
         **backend_kwargs,
     }
     if ax is None:
@@ -40,6 +42,7 @@ def plot_violin(
             len(plotters),
             rows,
             cols,
+            sharex=sharex,
             sharey=sharey,
             figsize=figsize,
             squeeze=False,
@@ -54,17 +57,30 @@ def plot_violin(
     ):
         val = x.flatten()
         if val[0].dtype.kind == "i":
-            cat_hist(val, shade, ax_, **kwargs_shade)
+            dens = cat_hist(val, rug, shade, ax_, **shade_kwargs)
         else:
-            _violinplot(val, shade, bw, ax_, **kwargs_shade)
+            dens = _violinplot(val, rug, shade, bw, ax_, **shade_kwargs)
+
+        if rug:
+            rug_x = -np.abs(np.random.normal(scale=max(dens) / 3.5, size=len(val)))
+            ax_.scatter(rug_x, val, **rug_kwargs)
 
         per = np.percentile(val, [25, 75, 50])
         hpd_intervals = hpd(val, credible_interval, multimodal=False)
 
         if quartiles:
-            ax_.line([0, 0], per[:2], line_width=linewidth * 3, line_color="black")
-        ax_.line([0, 0], hpd_intervals, line_width=linewidth, line_color="black")
-        ax_.circle(0, per[-1])
+            ax_.line(
+                [0, 0], per[:2], line_width=linewidth * 3, line_color="black", line_cap="round"
+            )
+        ax_.line([0, 0], hpd_intervals, line_width=linewidth, line_color="black", line_cap="round")
+        ax_.circle(
+            0,
+            per[-1],
+            line_color="white",
+            fill_color="white",
+            size=linewidth * 1.5,
+            line_width=linewidth,
+        )
 
         _title = Title()
         _title.text = make_label(var_name, selection)
@@ -73,42 +89,49 @@ def plot_violin(
         ax_.xaxis.minor_tick_line_color = None
         ax_.xaxis.major_label_text_font_size = "0pt"
 
-    if backend_show(show):
-        grid = gridplot(ax.tolist(), toolbar_location="above")
-        bkp.show(grid)
+    show_layout(ax, show)
 
     return ax
 
 
-def _violinplot(val, shade, bw, ax, **kwargs_shade):
+def _violinplot(val, rug, shade, bw, ax, **shade_kwargs):
     """Auxiliary function to plot violinplots."""
     density, low_b, up_b = _fast_kde(val, bw=bw)
     x = np.linspace(low_b, up_b, len(density))
 
-    x = np.concatenate([x, x[::-1]])
-    density = np.concatenate([-density, density[::-1]])
+    if not rug:
+        x = np.concatenate([x, x[::-1]])
+        density = np.concatenate([-density, density[::-1]])
 
-    ax.patch(density, x, fill_alpha=shade, line_width=0, **kwargs_shade)
+    ax.harea(y=x, x1=density, x2=np.zeros_like(density), fill_alpha=shade, **shade_kwargs)
+
+    return density
 
 
-def cat_hist(val, shade, ax, **kwargs_shade):
+def cat_hist(val, rug, shade, ax, **shade_kwargs):
     """Auxiliary function to plot discrete-violinplots."""
     bins = get_bins(val)
     _, binned_d, _ = histogram(val, bins=bins)
 
     bin_edges = np.linspace(np.min(val), np.max(val), len(bins))
-    centers = 0.5 * (bin_edges + np.roll(bin_edges, 1))[:-1]
     heights = np.diff(bin_edges)
+    centers = bin_edges[:-1] + heights.mean() / 2
+    right = 0.5 * binned_d
 
-    lefts = -0.5 * binned_d
+    if rug:
+        left = 0
+    else:
+        left = -right
 
     ax.hbar(
         y=centers,
-        left=lefts,
-        right=-lefts,
+        left=left,
+        right=right,
         height=heights,
         fill_alpha=shade,
         line_alpha=shade,
         line_color=None,
-        **kwargs_shade
+        **shade_kwargs
     )
+
+    return binned_d
