@@ -4,11 +4,12 @@ import matplotlib.pyplot as plt
 import numpy as np
 
 from . import backend_show
-from ...kdeplot import plot_kde, _fast_kde
+from ...kdeplot import plot_kde
 from ...plot_utils import (
     make_label,
     _create_axes_grid,
     get_bins,
+    _fast_kde,
 )
 from ....stats.stats_utils import histogram
 
@@ -22,7 +23,7 @@ def plot_ppc(
     animated,
     obs_plotters,
     pp_plotters,
-    posterior_predictive,
+    predictive_dataset,
     pp_sample_ix,
     kind,
     alpha,
@@ -33,6 +34,7 @@ def plot_ppc(
     jitter,
     total_pp_samples,
     legend,
+    group,
     markersize,
     animation_kwargs,
     num_pp_samples,
@@ -60,7 +62,7 @@ def plot_ppc(
     for i, ax_i in enumerate(axes):
         var_name, selection, obs_vals = obs_plotters[i]
         pp_var_name, _, pp_vals = pp_plotters[i]
-        dtype = posterior_predictive[pp_var_name].dtype.kind
+        dtype = predictive_dataset[pp_var_name].dtype.kind
 
         # flatten non-specified dimensions
         obs_vals = obs_vals.flatten()
@@ -71,7 +73,9 @@ def plot_ppc(
             plot_kwargs = {"color": "C5", "alpha": alpha, "linewidth": 0.5 * linewidth}
             if dtype == "i":
                 plot_kwargs["drawstyle"] = "steps-pre"
-            ax_i.plot([], color="C5", label="Posterior predictive {}".format(pp_var_name))
+            ax_i.plot(
+                [], color="C5", label="{} predictive {}".format(group.capitalize(), pp_var_name)
+            )
 
             if dtype == "f":
                 plot_kde(
@@ -125,6 +129,7 @@ def plot_ppc(
                         ax_i.plot(x_s, y_s, **plot_kwargs)
 
             if mean:
+                label = "{} predictive mean {}".format(group.capitalize(), pp_var_name)
                 if dtype == "f":
                     rep = len(pp_densities)
                     len_density = len(pp_densities[0])
@@ -142,7 +147,7 @@ def plot_ppc(
                         linestyle="--",
                         linewidth=linewidth,
                         zorder=2,
-                        label="Posterior predictive mean {}".format(pp_var_name),
+                        label=label,
                     )
                 else:
                     vals = pp_vals.flatten()
@@ -154,7 +159,7 @@ def plot_ppc(
                         hist,
                         color="C0",
                         linewidth=linewidth,
-                        label="Posterior predictive mean {}".format(pp_var_name),
+                        label=label,
                         zorder=2,
                         linestyle="--",
                         drawstyle=plot_kwargs["drawstyle"],
