@@ -1,12 +1,14 @@
 """Matplotlib traceplot."""
+from itertools import cycle
 
+import warnings
 import matplotlib.pyplot as plt
 from matplotlib.lines import Line2D
 import numpy as np
 
 from . import backend_kwarg_defaults, backend_show
 from ...distplot import plot_dist
-from ...plot_utils import _scale_fig_size, get_bins, make_label
+from ...plot_utils import _scale_fig_size, get_bins, make_label, format_coords_as_labels
 
 
 def plot_trace(
@@ -16,7 +18,9 @@ def plot_trace(
     figsize,
     rug,
     lines,
+    compact_prop,
     combined,
+    chain_prop,
     legend,
     plot_kwargs,
     fill_kwargs,
@@ -25,7 +29,7 @@ def plot_trace(
     trace_kwargs,
     plotters,
     divergence_data,
-    colors,
+    axes,
     backend_kwargs,
     show,
 ):
@@ -48,8 +52,8 @@ def plot_trace(
     rug : bool
         If True adds a rugplot. Defaults to False. Ignored for 2D KDE. Only affects continuous
         variables.
-    lines : tuple
-        Tuple of (var_name, {'coord': selection}, [line, positions]) to be overplotted as
+    lines : tuple or list
+        list of tuple of (var_name, {'coord': selection}, [line_positions]) to be overplotted as
         vertical lines on the density and horizontal lines on the trace.
     combined : bool
         Flag for combining multiple chains into a single line. If False (default), chains will be
@@ -122,18 +126,37 @@ def plot_trace(
     trace_kwargs.setdefault("linewidth", linewidth)
     plot_kwargs.setdefault("linewidth", linewidth)
 
-    _, axes = plt.subplots(len(plotters), 2, squeeze=False, figsize=figsize, **backend_kwargs)
+    if axes is None:
+        _, axes = plt.subplots(len(plotters), 2, squeeze=False, figsize=figsize, **backend_kwargs)
+
+    # Check the input for lines
+    if lines is not None:
+        all_var_names = set(plotter[0] for plotter in plotters)
+
+        invalid_var_names = set()
+        for line in lines:
+            if line[0] not in all_var_names:
+                invalid_var_names.add(line[0])
+        if invalid_var_names:
+            warnings.warn(
+                "A valid var_name should be provided, found {} expected from {}".format(
+                    invalid_var_names, all_var_names
+                )
+            )
 
     for idx, (var_name, selection, value) in enumerate(plotters):
         value = np.atleast_2d(value)
 
         if len(value.shape) == 2:
+            if compact_prop:
+                plot_kwargs[compact_prop[0]] = compact_prop[1][0]
+                trace_kwargs[compact_prop[0]] = compact_prop[1][0]
             _plot_chains_mpl(
                 axes,
                 idx,
                 value,
                 data,
-                colors,
+                chain_prop,
                 combined,
                 xt_labelsize,
                 rug,
@@ -143,15 +166,34 @@ def plot_trace(
                 fill_kwargs,
                 rug_kwargs,
             )
+            if compact_prop:
+                plot_kwargs.pop(compact_prop[0])
+                trace_kwargs.pop(compact_prop[0])
         else:
+            sub_data = data[var_name].sel(**selection)
+            legend_labels = format_coords_as_labels(sub_data, skip_dims=("chain", "draw"))
+            legend_title = ", ".join(
+                [
+                    "{}".format(coord_name)
+                    for coord_name in sub_data.coords
+                    if coord_name not in {"chain", "draw"}
+                ]
+            )
             value = value.reshape((value.shape[0], value.shape[1], -1))
-            for sub_idx in range(value.shape[2]):
+            compact_prop_cycle = cycle(compact_prop[1])
+            handles = []
+            for sub_idx, label, prop in zip(
+                range(value.shape[2]), legend_labels, compact_prop_cycle
+            ):
+                if compact_prop:
+                    plot_kwargs[compact_prop[0]] = prop
+                    trace_kwargs[compact_prop[0]] = prop
                 _plot_chains_mpl(
                     axes,
                     idx,
                     value[..., sub_idx],
                     data,
-                    colors,
+                    chain_prop,
                     combined,
                     xt_labelsize,
                     rug,
@@ -161,6 +203,16 @@ def plot_trace(
                     fill_kwargs,
                     rug_kwargs,
                 )
+                if legend:
+                    handles.append(
+                        Line2D(
+                            [], [], label=label, **{chain_prop[0]: chain_prop[1][0]}, **plot_kwargs
+                        )
+                    )
+            if legend:
+                axes[idx, 0].legend(handles=handles, title=legend_title)
+            plot_kwargs.pop(compact_prop[0], None)
+            trace_kwargs.pop(compact_prop[0], None)
 
         if value[0].dtype.kind == "i":
             xticks = get_bins(value)
@@ -219,6 +271,10 @@ def plot_trace(
                 line_values = [vlines]
             else:
                 line_values = np.atleast_1d(vlines).ravel()
+                if not np.issubdtype(line_values.dtype, np.number):
+                    raise ValueError(
+                        "line-positions should be numeric, found {}".format(line_values)
+                    )
             axes[idx, 0].vlines(line_values, *ylims[0], colors="black", linewidth=1.5, alpha=0.75)
             axes[idx, 1].hlines(
                 line_values, *xlims[1], colors="black", linewidth=1.5, alpha=trace_kwargs["alpha"]
@@ -227,12 +283,18 @@ def plot_trace(
         axes[idx, 1].set_xlim(left=data.draw.min(), right=data.draw.max())
         axes[idx, 1].set_ylim(*ylims[1])
     if legend:
+        legend_kwargs = trace_kwargs if combined else plot_kwargs
         handles = [
-            Line2D([], [], color=color, label=chain_id)
-            for chain_id, color in zip(data.chain.values, colors)
+            Line2D([], [], label=chain_id, **{chain_prop[0]: prop}, **legend_kwargs)
+            for chain_id, prop in zip(data.chain.values, chain_prop[1])
         ]
         if combined:
-            handles.insert(0, Line2D([], [], color=colors[-1], label="combined"))
+            handles.insert(
+                0,
+                Line2D(
+                    [], [], label="combined", **{chain_prop[0]: chain_prop[1][-1]}, **plot_kwargs
+                ),
+            )
         axes[0, 1].legend(handles=handles, title="chain")
 
     if backend_show(show):
@@ -246,7 +308,7 @@ def _plot_chains_mpl(
     idx,
     value,
     data,
-    colors,
+    chain_prop,
     combined,
     xt_labelsize,
     rug,
@@ -257,10 +319,12 @@ def _plot_chains_mpl(
     rug_kwargs,
 ):
     for chain_idx, row in enumerate(value):
-        axes[idx, 1].plot(data.draw.values, row, color=colors[chain_idx], **trace_kwargs)
+        axes[idx, 1].plot(
+            data.draw.values, row, **{chain_prop[0]: chain_prop[1][chain_idx]}, **trace_kwargs
+        )
 
         if not combined:
-            plot_kwargs["color"] = colors[chain_idx]
+            plot_kwargs[chain_prop[0]] = chain_prop[1][chain_idx]
             plot_dist(
                 values=row,
                 textsize=xt_labelsize,
@@ -271,12 +335,12 @@ def _plot_chains_mpl(
                 fill_kwargs=fill_kwargs,
                 rug_kwargs=rug_kwargs,
                 backend="matplotlib",
-                backend_kwargs={},
                 show=False,
             )
+            plot_kwargs.pop(chain_prop[0])
 
     if combined:
-        plot_kwargs["color"] = colors[-1]
+        plot_kwargs[chain_prop[0]] = chain_prop[1][-1]
         plot_dist(
             values=value.flatten(),
             textsize=xt_labelsize,
@@ -287,6 +351,6 @@ def _plot_chains_mpl(
             fill_kwargs=fill_kwargs,
             rug_kwargs=rug_kwargs,
             backend="matplotlib",
-            backend_kwargs={},
             show=False,
         )
+        plot_kwargs.pop(chain_prop[0])
