@@ -12,8 +12,8 @@ from .plot_utils import (
     default_grid,
     get_plotting_function,
 )
-from ..utils import _var_names
 from ..rcparams import rcParams
+from ..utils import _var_names
 
 
 # pylint:disable-msg=too-many-function-args
@@ -22,8 +22,9 @@ def plot_density(
     group="posterior",
     data_labels=None,
     var_names=None,
-    credible_interval=0.94,
-    point_estimate="mean",
+    transform=None,
+    credible_interval=None,
+    point_estimate="auto",
     colors="cycle",
     outline=True,
     hpd_markers="",
@@ -58,11 +59,13 @@ def plot_density(
         List of variables to plot.  If multiple datasets are supplied and var_names is not None,
         will print the same set of variables for each dataset.  Defaults to None, which results in
         all the variables being plotted.
+    transform : callable
+        Function to transform data (defaults to None i.e. the identity function)
     credible_interval : float
         Credible intervals. Should be in the interval (0, 1]. Defaults to 0.94.
     point_estimate : Optional[str]
-        Plot point estimate per variable. Values should be 'mean', 'median' or None.
-        Defaults to 'mean'.
+        Plot point estimate per variable. Values should be 'mean', 'median', 'mode' or None.
+        Defaults to 'auto' i.e. it falls back to default set in rcParams.
     colors : Optional[Union[List[str],str]]
         List with valid matplotlib colors, one color per model. Alternative a string can be passed.
         If the string is `cycle`, it will automatically choose a color per model from matplotlib's
@@ -147,18 +150,14 @@ def plot_density(
 
         >>> az.plot_density([centered, non_centered], var_names=["mu"], bw=.9)
     """
+    if transform is not None:
+        data = transform(data)
     if not isinstance(data, (list, tuple)):
         datasets = [convert_to_dataset(data, group=group)]
     else:
         datasets = [convert_to_dataset(datum, group=group) for datum in data]
 
     var_names = _var_names(var_names, datasets)
-
-    if point_estimate not in ("mean", "median", None):
-        raise ValueError(
-            "Point estimate should be 'mean'," "median' or None, not {}".format(point_estimate)
-        )
-
     n_data = len(datasets)
 
     if data_labels is None:
@@ -182,8 +181,11 @@ def plot_density(
     elif isinstance(colors, str):
         colors = [colors for _ in range(n_data)]
 
-    if not 1 >= credible_interval > 0:
-        raise ValueError("The value of credible_interval should be in the interval (0, 1]")
+    if credible_interval is None:
+        credible_interval = rcParams["stats.credible_interval"]
+    else:
+        if not 1 >= credible_interval > 0:
+            raise ValueError("The value of credible_interval should be in the interval (0, 1]")
 
     to_plot = [list(xarray_var_iter(data, var_names, combined=True)) for data in datasets]
     all_labels = []
@@ -194,7 +196,7 @@ def plot_density(
             label = make_label(var_name, selection)
             if label not in all_labels:
                 all_labels.append(label)
-    length_plotters = max(length_plotters)
+    length_plotters = len(all_labels)
     max_plots = rcParams["plot.max_subplots"]
     max_plots = length_plotters if max_plots is None else max_plots
     if length_plotters > max_plots:
@@ -202,7 +204,7 @@ def plot_density(
             "rcParams['plot.max_subplots'] ({max_plots}) is smaller than the number "
             "of variables to plot ({len_plotters}) in plot_density, generating only "
             "{max_plots} plots".format(max_plots=max_plots, len_plotters=length_plotters),
-            SyntaxWarning,
+            UserWarning,
         )
         all_labels = all_labels[:max_plots]
         to_plot = [
@@ -245,6 +247,10 @@ def plot_density(
         show=show,
     )
 
+    if backend is None:
+        backend = rcParams["plot.backend"]
+    backend = backend.lower()
+
     if backend == "bokeh":
 
         plot_density_kwargs["line_width"] = plot_density_kwargs.pop("linewidth")
