@@ -5,7 +5,13 @@ import matplotlib.cm as cm
 from matplotlib.lines import Line2D
 
 from ..data import convert_to_inference_data
-from .plot_utils import get_coords, format_coords_as_labels, color_from_dim, get_plotting_function
+from .plot_utils import (
+    get_coords,
+    format_coords_as_labels,
+    color_from_dim,
+    get_plotting_function,
+    matplotlib_kwarg_dealiaser,
+)
 from ..stats import waic, loo, ELPDData
 from ..rcparams import rcParams
 
@@ -21,7 +27,7 @@ def plot_elpd(
     threshold=None,
     ax=None,
     ic=None,
-    scale="deviance",
+    scale=None,
     plot_kwargs=None,
     backend=None,
     backend_kwargs=None,
@@ -34,7 +40,7 @@ def plot_elpd(
     ----------
     compare_dict : mapping, str -> ELPDData or InferenceData
         A dictionary mapping the model name to the object containing its inference data or
-        the result of `waic`/`loo` functions.
+        the result of `loo`/`waic` functions.
         Refer to az.convert_to_inference_data for details on possible dict items
     color : str or array_like, optional
         Colors of the scatter plot, if color is a str all dots will have the same color,
@@ -57,14 +63,14 @@ def plot_elpd(
     ax: axes, optional
         Matplotlib axes or bokeh figures.
     ic : str, optional
-        Information Criterion (WAIC or LOO) used to compare models. Defaults to
+        Information Criterion (PSIS-LOO `loo`, WAIC `waic`) used to compare models. Defaults to
         ``rcParams["stats.information_criterion"]``. Only taken
         into account when input is InferenceData.
     scale : str, optional
-        scale argument passed to az.waic or az.loo, see their docs for details. Only taken
+        scale argument passed to az.loo or az.waic, see their docs for details. Only taken
         into account when input is InferenceData.
     plot_kwargs : dicts, optional
-        Additional keywords passed to ax.plot
+        Additional keywords passed to ax.scatter
     ax: axes, optional
         Matplotlib axes or bokeh figures.
     backend: str, optional
@@ -81,7 +87,7 @@ def plot_elpd(
 
     Examples
     --------
-    Compare pointwise WAIC for centered and non centered models of the 8school problem
+    Compare pointwise PSIS-LOO for centered and non centered models of the 8-schools problem
     using matplotlib.
 
     .. plot::
@@ -107,15 +113,16 @@ def plot_elpd(
         )
 
     """
-    valid_ics = ["waic", "loo"]
+    valid_ics = ["loo", "waic"]
     ic = rcParams["stats.information_criterion"] if ic is None else ic.lower()
+    scale = rcParams["stats.ic_scale"] if scale is None else scale.lower()
     if ic not in valid_ics:
         raise ValueError(
             ("Information Criteria type {} not recognized." "IC must be in {}").format(
                 ic, valid_ics
             )
         )
-    ic_fun = waic if ic == "waic" else loo
+    ic_fun = loo if ic == "loo" else waic
     handles = None
     # Make sure all object are ELPDData
     for k, item in compare_dict.items():
@@ -134,16 +141,21 @@ def plot_elpd(
                 set(scales)
             )
         )
-    backend = rcParams["plot.backend"] if backend is None else backend.lower()
+
+    if backend is None:
+        backend = rcParams["plot.backend"]
+    backend = backend.lower()
+
     numvars = len(compare_dict)
     models = list(compare_dict.keys())
 
     if coords is None:
         coords = {}
 
-    if plot_kwargs is None:
-        plot_kwargs = {}
-    plot_kwargs.setdefault("marker", "+")
+    plot_kwargs = matplotlib_kwarg_dealiaser(plot_kwargs, "scatter")
+
+    if backend == "bokeh":
+        plot_kwargs.setdefault("marker", rcParams["plot.bokeh.marker"])
 
     pointwise_data = [
         get_coords(compare_dict[model]["{}_i".format(ic)], coords) for model in models
