@@ -1,10 +1,11 @@
 """Plot energy transition distribution in HMC inference."""
 from itertools import cycle
-from matplotlib.pyplot import rcParams
+from matplotlib.pyplot import rcParams as mpl_rcParams
 import numpy as np
 
 from ..data import convert_to_dataset
-from .plot_utils import _scale_fig_size, get_plotting_function
+from .plot_utils import _scale_fig_size, get_plotting_function, matplotlib_kwarg_dealiaser
+from ..rcparams import rcParams
 
 
 def plot_energy(
@@ -92,16 +93,14 @@ def plot_energy(
     """
     energy = convert_to_dataset(data, group="sample_stats").energy.values
 
-    if fill_kwargs is None:
-        fill_kwargs = {}
-
-    if plot_kwargs is None:
-        plot_kwargs = {}
+    fill_kwargs = matplotlib_kwarg_dealiaser(fill_kwargs, "hexbin")
+    types = "hist" if kind in {"hist", "histogram"} else "plot"
+    plot_kwargs = matplotlib_kwarg_dealiaser(plot_kwargs, types)
 
     figsize, _, _, xt_labelsize, linewidth, _ = _scale_fig_size(figsize, textsize, 1, 1)
 
     _colors = [
-        prop for _, prop in zip(range(10), cycle(rcParams["axes.prop_cycle"].by_key()["color"]))
+        prop for _, prop in zip(range(10), cycle(mpl_rcParams["axes.prop_cycle"].by_key()["color"]))
     ]
     if (fill_color[0].startswith("C") and len(fill_color[0]) == 2) and (
         fill_color[1].startswith("C") and len(fill_color[1]) == 2
@@ -136,6 +135,10 @@ def plot_energy(
         show=show,
     )
 
+    if backend is None:
+        backend = rcParams["plot.backend"]
+    backend = backend.lower()
+
     if backend == "bokeh":
 
         plot_energy_kwargs.pop("xt_labelsize")
