@@ -2,6 +2,7 @@
 from ..data import convert_to_dataset
 from .plot_utils import get_coords, get_plotting_function
 from ..utils import _var_names
+from ..rcparams import rcParams
 
 
 def plot_forest(
@@ -9,9 +10,10 @@ def plot_forest(
     kind="forestplot",
     model_names=None,
     var_names=None,
+    transform=None,
     coords=None,
     combined=False,
-    credible_interval=0.94,
+    credible_interval=None,
     rope=None,
     quartiles=True,
     ess=False,
@@ -23,9 +25,11 @@ def plot_forest(
     ridgeplot_alpha=None,
     ridgeplot_overlap=2,
     ridgeplot_kind="auto",
+    ridgeplot_quantiles=None,
     figsize=None,
     ax=None,
     backend=None,
+    backend_config=None,
     backend_kwargs=None,
     show=None,
 ):
@@ -47,6 +51,8 @@ def plot_forest(
     var_names: list[str], optional
         List of variables to plot (defaults to None, which results in all
         variables plotted)
+    transform : callable
+        Function to transform data (defaults to None i.e.the identity function)
     coords : dict, optional
         Coordinates of var_names to be plotted. Passed to `Dataset.sel`
     combined : bool
@@ -85,12 +91,17 @@ def plot_forest(
     ridgeplot_kind : string
         By default ("auto") continuous variables are plotted using KDEs and discrete ones using
         histograms. To override this use "hist" to plot histograms and "density" for KDEs
+    ridgeplot_quantiles : list
+        Quantiles in ascending order used to segment the KDE. Use [.25, .5, .75] for quartiles.
+        Defaults to None.
     figsize : tuple
         Figure size. If None it will be defined automatically.
     ax: axes, optional
         Matplotlib axes or bokeh figures.
     backend: str, optional
         Select plotting backend {"matplotlib","bokeh"}. Default "matplotlib".
+    backend_config: dict, optional
+        Currently specifies the bounds to use for bokeh axes. Defaults to value set in rcParams.
     backend_kwargs: bool, optional
         These are kwargs specific to the backend being used. For additional documentation
         check the plotting method of the backend.
@@ -134,6 +145,8 @@ def plot_forest(
     """
     if not isinstance(data, (list, tuple)):
         data = [data]
+    if transform is not None:
+        data = transform(data)
 
     if coords is None:
         coords = {}
@@ -154,6 +167,12 @@ def plot_forest(
         ncols += 1
         width_ratios.append(1)
 
+    if credible_interval is None:
+        credible_interval = rcParams["stats.credible_interval"]
+    else:
+        if not 1 >= credible_interval > 0:
+            raise ValueError("The value of credible_interval should be in the interval (0, 1]")
+
     plot_forest_kwargs = dict(
         ax=ax,
         datasets=datasets,
@@ -173,6 +192,7 @@ def plot_forest(
         ridgeplot_overlap=ridgeplot_overlap,
         ridgeplot_alpha=ridgeplot_alpha,
         ridgeplot_kind=ridgeplot_kind,
+        ridgeplot_quantiles=ridgeplot_quantiles,
         textsize=textsize,
         ess=ess,
         r_hat=r_hat,
@@ -180,6 +200,13 @@ def plot_forest(
         show=show,
     )
 
+    if backend is None:
+        backend = rcParams["plot.backend"]
+    backend = backend.lower()
+
+    if backend == "bokeh":
+        plot_forest_kwargs.update(backend_config=backend_config)
+
     # TODO: Add backend kwargs
     plot = get_plotting_function("plot_forest", "forestplot", backend)
     axes = plot(**plot_forest_kwargs)
