@@ -4,13 +4,14 @@ from scipy.interpolate import griddata
 from scipy.signal import savgol_filter
 
 from ..stats import hpd
-from .plot_utils import get_plotting_function
+from .plot_utils import get_plotting_function, matplotlib_kwarg_dealiaser
+from ..rcparams import rcParams
 
 
 def plot_hpd(
     x,
     y,
-    credible_interval=0.94,
+    credible_interval=None,
     color="C1",
     circular=False,
     smooth=True,
@@ -63,13 +64,11 @@ def plot_hpd(
     -------
     axes : matplotlib axes or bokeh figures
     """
-    if plot_kwargs is None:
-        plot_kwargs = {}
+    plot_kwargs = matplotlib_kwarg_dealiaser(plot_kwargs, "plot")
     plot_kwargs.setdefault("color", color)
     plot_kwargs.setdefault("alpha", 0)
 
-    if fill_kwargs is None:
-        fill_kwargs = {}
+    fill_kwargs = matplotlib_kwarg_dealiaser(fill_kwargs, "hexbin")
     fill_kwargs.setdefault("color", color)
     fill_kwargs.setdefault("alpha", 0.5)
 
@@ -88,6 +87,12 @@ def plot_hpd(
         new_shape = tuple([-1] + list(x_shape))
         y = y.reshape(new_shape)
 
+    if credible_interval is None:
+        credible_interval = rcParams["stats.credible_interval"]
+    else:
+        if not 1 >= credible_interval > 0:
+            raise ValueError("The value of credible_interval should be in the interval (0, 1]")
+
     hpd_ = hpd(y, credible_interval=credible_interval, circular=circular, multimodal=False)
 
     if smooth:
@@ -96,6 +101,7 @@ def plot_hpd(
         smooth_kwargs.setdefault("window_length", 55)
         smooth_kwargs.setdefault("polyorder", 2)
         x_data = np.linspace(x.min(), x.max(), 200)
+        x_data[0] = (x_data[0] + x_data[1]) / 2
         hpd_interp = griddata(x, hpd_, x_data)
         y_data = savgol_filter(hpd_interp, axis=0, **smooth_kwargs)
     else:
@@ -113,6 +119,10 @@ def plot_hpd(
         show=show,
     )
 
+    if backend is None:
+        backend = rcParams["plot.backend"]
+    backend = backend.lower()
+
     # TODO: Add backend kwargs
     plot = get_plotting_function("plot_hpd", "hpdplot", backend)
     ax = plot(**hpdplot_kwargs)
