@@ -1,12 +1,21 @@
 """Joint scatter plot of two variables."""
 from ..data import convert_to_dataset
-from .plot_utils import _scale_fig_size, xarray_var_iter, get_coords, get_plotting_function
+from .plot_utils import (
+    _scale_fig_size,
+    xarray_var_iter,
+    get_coords,
+    get_plotting_function,
+    matplotlib_kwarg_dealiaser,
+)
+from ..rcparams import rcParams
 from ..utils import _var_names
 
 
 def plot_joint(
     data,
+    group="posterior",
     var_names=None,
+    transform=None,
     coords=None,
     figsize=None,
     textsize=None,
@@ -29,8 +38,13 @@ def plot_joint(
     data : obj
         Any object that can be converted to an az.InferenceData object
         Refer to documentation of az.convert_to_dataset for details
-    var_names : Iter of 2 e.g. (var_1, var_2)
-        Variables to be plotted, two variables are required.
+    group : str, optional
+        Specifies which InferenceData group should be plotted. Defaults to ‘posterior’.
+    var_names : str or iterable of str
+        Variables to be plotted. iter of two variables or one variable (with subset having
+        exactly 2 dimensions) are required.
+    transform : callable
+        Function to transform data (defaults to None i.e. the identity function)
     coords : mapping, optional
         Coordinates of var_names to be plotted. Passed to `Dataset.sel`
     figsize : tuple
@@ -130,7 +144,10 @@ def plot_joint(
             ("Plot type {} not recognized." "Plot type must be in {}").format(kind, valid_kinds)
         )
 
-    data = convert_to_dataset(data, group="posterior")
+    data = convert_to_dataset(data, group=group)
+
+    if transform is not None:
+        data = transform(data)
 
     if coords is None:
         coords = {}
@@ -146,8 +163,13 @@ def plot_joint(
 
     figsize, ax_labelsize, _, xt_labelsize, linewidth, _ = _scale_fig_size(figsize, textsize)
 
-    if joint_kwargs is None:
-        joint_kwargs = {}
+    if kind == "kde":
+        types = "plot"
+    elif kind == "scatter":
+        types = "scatter"
+    else:
+        types = "hexbin"
+    joint_kwargs = matplotlib_kwarg_dealiaser(joint_kwargs, types)
 
     if marginal_kwargs is None:
         marginal_kwargs = {}
@@ -170,6 +192,10 @@ def plot_joint(
         show=show,
     )
 
+    if backend is None:
+        backend = rcParams["plot.backend"]
+    backend = backend.lower()
+
     if backend == "bokeh":
 
         plot_joint_kwargs.pop("ax_labelsize")
