@@ -4,6 +4,7 @@ from scipy.stats.mstats import rankdata
 
 from ..data import convert_to_dataset
 from .plot_utils import _scale_fig_size, xarray_to_ndarray, get_coords, get_plotting_function
+from ..rcparams import rcParams
 from ..utils import _var_names, _numba_var
 from ..stats.stats_utils import stats_variance_2d as svar
 
@@ -21,6 +22,7 @@ def plot_parallel(
     ax=None,
     norm_method=None,
     backend=None,
+    backend_config=None,
     backend_kwargs=None,
     show=None,
 ):
@@ -60,6 +62,8 @@ def plot_parallel(
         Defaults to none.
     backend: str, optional
         Select plotting backend {"matplotlib","bokeh"}. Default "matplotlib".
+    backend_config: dict, optional
+        Currently specifies the bounds to use for bokeh axes. Defaults to value set in rcParams.
     backend_kwargs: bool, optional
         These are kwargs specific to the backend being used. For additional documentation
         check the plotting method of the backend.
@@ -143,6 +147,10 @@ def plot_parallel(
         show=show,
     )
 
+    if backend is None:
+        backend = rcParams["plot.backend"]
+    backend = backend.lower()
+
     if backend == "bokeh":
 
         parallel_kwargs.pop("textsize")
@@ -151,6 +159,7 @@ def plot_parallel(
         parallel_kwargs.pop("colord")
         parallel_kwargs.pop("colornd")
         parallel_kwargs.pop("shadend")
+        parallel_kwargs.update(backend_config=backend_config)
 
     # TODO: Add backend kwargs
     plot = get_plotting_function("plot_parallel", "parallelplot", backend)
