@@ -1,18 +1,26 @@
 """Utilities for plotting."""
 import warnings
+from typing import Dict, Any
 from itertools import product, tee
 import importlib
+from scipy.signal import convolve, convolve2d
+from scipy.sparse import coo_matrix
+from scipy.signal.windows import gaussian
+from scipy.stats import mode
 
 import packaging
 import numpy as np
 import matplotlib.pyplot as plt
 import matplotlib as mpl
+import matplotlib.cbook as cbook
 import xarray as xr
 
 
-from ..utils import conditional_jit
+from ..utils import conditional_jit, _stack
 from ..rcparams import rcParams
 
+KwargSpec = Dict[str, Any]
+
 
 def make_2d(ary):
     """Convert any array into a 2d numpy array.
@@ -215,26 +223,23 @@ def _create_axes_grid(length_plotters, rows, cols, backend=None, backend_kwargs=
     if backend_kwargs is None:
         backend_kwargs = {}
 
+    if backend is None:
+        backend = rcParams["plot.backend"]
+    backend = backend.lower()
+
     if backend == "bokeh":
         from bokeh.plotting import figure
         from .backends.bokeh import backend_kwarg_defaults
 
         backend_kwargs = {
-            **backend_kwarg_defaults(
-                ("tools", "plot.bokeh.tools"),
-                ("output_backend", "plot.bokeh.output_backend"),
-                ("width", "plot.bokeh.figure.width"),
-                ("height", "plot.bokeh.figure.height"),
-                ("dpi", "plot.bokeh.figure.dpi"),
-            ),
+            **backend_kwarg_defaults(("dpi", "plot.bokeh.figure.dpi"),),
             **backend_kwargs,
         }
         dpi = backend_kwargs.pop("dpi")
         if "figsize" in kwargs:
             figsize = kwargs["figsize"]
-            figsize = int(figsize[0] * dpi / cols), int(figsize[1] * dpi / rows)
-            backend_kwargs["width"] = figsize[0]
-            backend_kwargs["height"] = figsize[1]
+            backend_kwargs.setdefault("width", int(figsize[0] * dpi / cols))
+            backend_kwargs.setdefault("height", int(figsize[1] * dpi / rows))
 
         sharex = kwargs.get("sharex", False)
         sharey = kwargs.get("sharey", False)
@@ -297,8 +302,9 @@ def make_label(var_name, selection, position="below"):
 
     selection : dict[Any] -> Any
         Coordinates of the variable
-    position : whether to position the coordinates' label "below" (default) or "beside" the name
-               of the variable
+    position : str
+        Whether to position the coordinates' label "below" (default) or "beside"
+        the name of the variable
 
     Returns
     -------
@@ -350,7 +356,6 @@ def round_num(n, round_to):
     return "{n:.{sig_figs}g}".format(n=n, sig_figs=sig_figs)
 
 
-@conditional_jit(forceobj=True)
 def purge_duplicates(list_in):
     """Remove duplicates from list while preserving order.
 
@@ -361,13 +366,19 @@ def purge_duplicates(list_in):
     Returns
     -------
     list
-        List of first occurences in order
+        List of first occurrences in order
     """
-    _list = []
-    for item in list_in:
-        if item not in _list:
-            _list.append(item)
-    return _list
+    # Algorithm taken from Stack Overflow,
+    # https://stackoverflow.com/questions/480214. Content by Georgy
+    # Skorobogatov (https://stackoverflow.com/users/7851470/georgy) and
+    # Markus Jarderot
+    # (https://stackoverflow.com/users/22364/markus-jarderot), licensed
+    # under CC-BY-SA 4.0.
+    # https://creativecommons.org/licenses/by-sa/4.0/.
+
+    seen = set()
+    seen_add = seen.add
+    return [x for x in list_in if not (x in seen or seen_add(x))]
 
 
 def _dims(data, var_name, skip_dims):
@@ -592,9 +603,21 @@ def color_from_dim(dataarray, dim_name):
     return colors, color_mapping
 
 
-def format_coords_as_labels(dataarray):
-    """Format 1d or multi-d dataarray coords as strings."""
-    coord_labels = dataarray.coords.to_index().values
+def format_coords_as_labels(dataarray, skip_dims=None):
+    """Format 1d or multi-d dataarray coords as strings.
+
+    Parameters
+    ----------
+    dataarray : xarray.DataArray
+        DataArray whose coordinates will be converted to labels.
+    skip_dims : str of list_like, optional
+        Dimensions whose values should not be included in the labels
+    """
+    if skip_dims is None:
+        coord_labels = dataarray.coords.to_index()
+    else:
+        coord_labels = dataarray.coords.to_index().droplevel(skip_dims).drop_duplicates()
+    coord_labels = coord_labels.values
     if isinstance(coord_labels[0], tuple):
         fmt = ", ".join(["{}" for _ in coord_labels[0]])
         coord_labels[:] = [fmt.format(*x) for x in coord_labels]
@@ -627,7 +650,7 @@ def filter_plotters_list(plotters, plot_kind):
             "{max_plots} plots".format(
                 max_plots=max_plots, len_plotters=len(plotters), plot_kind=plot_kind
             ),
-            SyntaxWarning,
+            UserWarning,
         )
         return plotters[:max_plots]
     return plotters
@@ -639,9 +662,12 @@ def get_plotting_function(plot_name, plot_module, backend):
         "mpl": "matplotlib",
         "bokeh": "bokeh",
         "matplotlib": "matplotlib",
-        None: "matplotlib",
     }
 
+    if backend is None:
+        backend = rcParams["plot.backend"]
+    backend = backend.lower()
+
     try:
         backend = _backend[backend]
     except KeyError:
@@ -673,3 +699,240 @@ def get_plotting_function(plot_name, plot_module, backend):
     plotting_method = getattr(module, plot_name)
 
     return plotting_method
+
+
+def calculate_point_estimate(point_estimate, values, bw):
+    """Validate and calculate the point estimate.
+
+    Parameters
+    ----------
+    point_estimate : Optional[str]
+        Plot point estimate per variable. Values should be 'mean', 'median', 'mode' or None.
+        Defaults to 'auto' i.e. it falls back to default set in rcParams.
+    values : 1-d array
+    bw : float
+        Bandwidth scaling factor. Should be larger than 0. The higher this number the smoother the
+        KDE will be. Defaults to 4.5 which is essentially the same as the Scott's rule of thumb
+        (the default used rule by SciPy).
+
+    Returns
+    -------
+    point_value : float
+        best estimate of data distribution
+    """
+    point_value = None
+    if point_estimate == "auto":
+        point_estimate = rcParams["plot.point_estimate"]
+    elif point_estimate not in ("mean", "median", "mode", None):
+        raise ValueError(
+            "Point estimate should be 'mean', 'median', 'mode' or None, not {}".format(
+                point_estimate
+            )
+        )
+    if point_estimate == "mean":
+        point_value = values.mean()
+    elif point_estimate == "mode":
+        if isinstance(values[0], float):
+            density, lower, upper = _fast_kde(values, bw=bw)
+            x = np.linspace(lower, upper, len(density))
+            point_value = x[np.argmax(density)]
+        else:
+            point_value = mode(values)[0][0]
+    elif point_estimate == "median":
+        point_value = np.median(values)
+
+    return point_value
+
+
+def _fast_kde(x, cumulative=False, bw=4.5, xmin=None, xmax=None):
+    """Fast Fourier transform-based Gaussian kernel density estimate (KDE).
+
+    The code was adapted from https://github.com/mfouesneau/faststats
+
+    Parameters
+    ----------
+    x : Numpy array or list
+    cumulative : bool
+        If true, estimate the cdf instead of the pdf
+    bw : float
+        Bandwidth scaling factor for the KDE. Should be larger than 0. The higher this number the
+        smoother the KDE will be. Defaults to 4.5 which is essentially the same as the Scott's rule
+        of thumb (the default rule used by SciPy).
+    xmin : float
+        Manually set lower limit.
+    xmax : float
+        Manually set upper limit.
+
+    Returns
+    -------
+    density: A gridded 1D KDE of the input points (x)
+    xmin: minimum value of x
+    xmax: maximum value of x
+    """
+    x = np.asarray(x, dtype=float)
+    x = x[np.isfinite(x)]
+    if x.size == 0:
+        warnings.warn("kde plot failed, you may want to check your data")
+        return np.array([np.nan]), np.nan, np.nan
+
+    len_x = len(x)
+    n_points = 200 if (xmin or xmax) is None else 500
+
+    if xmin is None:
+        xmin = np.min(x)
+    if xmax is None:
+        xmax = np.max(x)
+
+    assert np.min(x) >= xmin
+    assert np.max(x) <= xmax
+
+    log_len_x = np.log(len_x) * bw
+
+    n_bins = min(int(len_x ** (1 / 3) * log_len_x * 2), n_points)
+    if n_bins < 2:
+        warnings.warn("kde plot failed, you may want to check your data")
+        return np.array([np.nan]), np.nan, np.nan
+
+    hist, bin_edges = np.histogram(x, bins=n_bins, range=(xmin, xmax))
+    grid = hist / (hist.sum() * np.diff(bin_edges))
+
+    # _, grid, _ = histogram(x, n_bins, range_hist=(xmin, xmax))
+
+    scotts_factor = len_x ** (-0.2)
+    kern_nx = int(scotts_factor * 2 * np.pi * log_len_x)
+    kernel = gaussian(kern_nx, scotts_factor * log_len_x)
+
+    npad = min(n_bins, 2 * kern_nx)
+    grid = np.concatenate([grid[npad:0:-1], grid, grid[n_bins : n_bins - npad : -1]])
+    density = convolve(grid, kernel, mode="same", method="direct")[npad : npad + n_bins]
+    norm_factor = (2 * np.pi * log_len_x ** 2 * scotts_factor ** 2) ** 0.5
+
+    density /= norm_factor
+
+    if cumulative:
+        density = density.cumsum() / density.sum()
+
+    return density, xmin, xmax
+
+
+def _cov_1d(x):
+    x = x - x.mean(axis=0)
+    ddof = x.shape[0] - 1
+    return np.dot(x.T, x.conj()) / ddof
+
+
+def _cov(data):
+    if data.ndim == 1:
+        return _cov_1d(data)
+    elif data.ndim == 2:
+        x = data.astype(float)
+        avg, _ = np.average(x, axis=1, weights=None, returned=True)
+        ddof = x.shape[1] - 1
+        if ddof <= 0:
+            warnings.warn("Degrees of freedom <= 0 for slice", RuntimeWarning, stacklevel=2)
+            ddof = 0.0
+        x -= avg[:, None]
+        prod = _dot(x, x.T.conj())
+        prod *= np.true_divide(1, ddof)
+        prod = prod.squeeze()
+        prod += 1e-6 * np.eye(prod.shape[0])
+        return prod
+    else:
+        raise ValueError("{} dimension arrays are not supported".format(data.ndim))
+
+
+@conditional_jit(cache=True)
+def _dot(x, y):
+    return np.dot(x, y)
+
+
+def _fast_kde_2d(x, y, gridsize=(128, 128), circular=False):
+    """
+    2D fft-based Gaussian kernel density estimate (KDE).
+
+    The code was adapted from https://github.com/mfouesneau/faststats
+
+    Parameters
+    ----------
+    x : Numpy array or list
+    y : Numpy array or list
+    gridsize : tuple
+        Number of points used to discretize data. Use powers of 2 for fft optimization
+    circular: bool
+        If True, use circular boundaries. Defaults to False
+    Returns
+    -------
+    grid: A gridded 2D KDE of the input points (x, y)
+    xmin: minimum value of x
+    xmax: maximum value of x
+    ymin: minimum value of y
+    ymax: maximum value of y
+    """
+    x = np.asarray(x, dtype=float)
+    x = x[np.isfinite(x)]
+    y = np.asarray(y, dtype=float)
+    y = y[np.isfinite(y)]
+
+    xmin, xmax = x.min(), x.max()
+    ymin, ymax = y.min(), y.max()
+
+    len_x = len(x)
+    weights = np.ones(len_x)
+    n_x, n_y = gridsize
+
+    d_x = (xmax - xmin) / (n_x - 1)
+    d_y = (ymax - ymin) / (n_y - 1)
+
+    xyi = _stack(x, y).T
+    xyi -= [xmin, ymin]
+    xyi /= [d_x, d_y]
+    xyi = np.floor(xyi, xyi).T
+
+    scotts_factor = len_x ** (-1 / 6)
+    cov = _cov(xyi)
+    std_devs = np.diag(cov ** 0.5)
+    kern_nx, kern_ny = np.round(scotts_factor * 2 * np.pi * std_devs)
+
+    inv_cov = np.linalg.inv(cov * scotts_factor ** 2)
+
+    x_x = np.arange(kern_nx) - kern_nx / 2
+    y_y = np.arange(kern_ny) - kern_ny / 2
+    x_x, y_y = np.meshgrid(x_x, y_y)
+
+    kernel = _stack(x_x.flatten(), y_y.flatten())
+    kernel = _dot(inv_cov, kernel) * kernel
+    kernel = np.exp(-kernel.sum(axis=0) / 2)
+    kernel = kernel.reshape((int(kern_ny), int(kern_nx)))
+
+    boundary = "wrap" if circular else "symm"
+
+    grid = coo_matrix((weights, xyi), shape=(n_x, n_y)).toarray()
+    grid = convolve2d(grid, kernel, mode="same", boundary=boundary)
+
+    norm_factor = np.linalg.det(2 * np.pi * cov * scotts_factor ** 2)
+    norm_factor = len_x * d_x * d_y * norm_factor ** 0.5
+
+    grid /= norm_factor
+
+    return grid, xmin, xmax, ymin, ymax
+
+
+def matplotlib_kwarg_dealiaser(args, kind, backend="matplotlib"):
+    """De-aliase the kwargs passed to plots."""
+    if args is None:
+        return {}
+    matplotlib_kwarg_dealiaser_dict = {
+        "scatter": mpl.collections.PathCollection,
+        "plot": mpl.lines.Line2D,
+        "hist": mpl.patches.Patch,
+        "hexbin": mpl.collections.PolyCollection,
+        "hlines": mpl.collections.LineCollection,
+        "text": mpl.text.Text,
+        "contour": mpl.contour.ContourSet,
+        "pcolormesh": mpl.collections.QuadMesh,
+    }
+    if backend == "matplotlib":
+        return cbook.normalize_kwargs(
+            args, getattr(matplotlib_kwarg_dealiaser_dict[kind], "_alias_map", {})
+        )
+    return args
