@@ -11,12 +11,14 @@ from .plot_utils import (
     _sturges_formula,
     get_plotting_function,
 )
+from ..rcparams import rcParams
 from ..utils import _var_names
 
 
 def plot_rank(
     data,
     var_names=None,
+    transform=None,
     coords=None,
     bins=None,
     kind="bars",
@@ -50,6 +52,8 @@ def plot_rank(
         az.convert_to_dataset for details
     var_names : string or list of variable names
         Variables to be plotted
+    transform : callable
+        Function to transform data (defaults to None i.e.the identity function)
     coords : mapping, optional
         Coordinates of var_names to be plotted. Passed to `Dataset.sel`
     bins : None or passed to np.histogram
@@ -116,6 +120,8 @@ def plot_rank(
         >>> az.plot_rank(noncentered_data, var_names="mu", kind='vlines', axes=ax[1])
 
     """
+    if transform is not None:
+        data = transform(data)
     posterior_data = convert_to_dataset(data, group="posterior")
     if coords is not None:
         posterior_data = posterior_data.sel(**coords)
@@ -165,6 +171,10 @@ def plot_rank(
         show=show,
     )
 
+    if backend is None:
+        backend = rcParams["plot.backend"]
+    backend = backend.lower()
+
     if backend == "bokeh":
 
         rankplot_kwargs.pop("ax_labelsize")
