@@ -7,6 +7,7 @@ import pprint
 import logging
 import locale
 from collections.abc import MutableMapping
+import numpy as np
 
 _log = logging.getLogger(__name__)
 
@@ -49,6 +50,45 @@ def _make_validate_choice(accepted_values, allow_none=False, typeof=str):
     return validate_choice
 
 
+def _make_validate_choice_regex(accepted_values, accepted_values_regex, allow_none=False):
+    """Validate value is in accepted_values with regex.
+
+    Parameters
+    ----------
+    accepted_values : iterable
+        Iterable containing all accepted_values.
+    accepted_values_regex : iterable
+        Iterable containing all accepted_values with regex string.
+    allow_none: boolean, optional
+        Whether to accept ``None`` in addition to the values in ``accepted_values``.
+    typeof: type, optional
+        Type the values should be converted to.
+    """
+    # no blank lines allowed after function docstring by pydocstyle,
+    # but black requires white line before function
+
+    def validate_choice_regex(value):
+        if allow_none and (value is None or isinstance(value, str) and value.lower() == "none"):
+            return None
+        value = str(value)
+        if isinstance(value, str):
+            value = value.lower()
+
+        if value in accepted_values:
+            # Convert value to python boolean if string matches
+            value = {"true": True, "false": False}.get(value, value)
+            return value
+        elif any(re.match(pattern, value) for pattern in accepted_values_regex):
+            return value
+        raise ValueError(
+            "{} is not one of {} or in regex {}{}".format(
+                value, accepted_values, accepted_values_regex, " nor None" if allow_none else ""
+            )
+        )
+
+    return validate_choice_regex
+
+
 def _validate_positive_int(value):
     """Validate value is a natural number."""
     try:
@@ -78,6 +118,14 @@ def _validate_float(value):
     return value
 
 
+def _validate_float_or_none(value):
+    """Validate value is a float or None."""
+    if value is None or isinstance(value, str) and value.lower() == "none":
+        return None
+    else:
+        return _validate_float(value)
+
+
 def _validate_probability(value):
     """Validate a probability: a float between 0 and 1."""
     value = _validate_float(value)
@@ -93,30 +141,107 @@ def _validate_boolean(value):
     return value is True or value == "true"
 
 
+def _validate_bokeh_marker(value):
+    """Validate the markers."""
+    all_markers = (
+        "Asterisk",
+        "Circle",
+        "CircleCross",
+        "CircleX",
+        "Cross",
+        "Dash",
+        "Diamond",
+        "DiamondCross",
+        "Hex",
+        "InvertedTriangle",
+        "Square",
+        "SquareCross",
+        "SquareX",
+        "Triangle",
+        "X",
+    )
+    if value not in all_markers:
+        raise ValueError("{} is not one of {}".format(value, all_markers))
+    return value
+
+
+def make_iterable_validator(scalar_validator, length=None, allow_none=False, allow_auto=False):
+    """Validate value is an iterable datatype."""
+    # based on matplotlib's _listify_validator function
+
+    def validate_iterable(value):
+        if allow_none and (value is None or isinstance(value, str) and value.lower() == "none"):
+            return None
+        if isinstance(value, str):
+            if allow_auto and value.lower() == "auto":
+                return "auto"
+            value = tuple(v.strip("([ ])") for v in value.split(",") if v.strip())
+        if np.iterable(value) and not isinstance(value, (set, frozenset)):
+            val = tuple(scalar_validator(v) for v in value)
+            if length is not None and len(val) != length:
+                raise ValueError("Iterable must be of length: {}".format(length))
+            return val
+        raise ValueError("Only ordered iterable values are valid")
+
+    return validate_iterable
+
+
+_validate_bokeh_bounds = make_iterable_validator(  # pylint: disable=invalid-name
+    _validate_float_or_none, length=2, allow_none=True, allow_auto=True
+)
+
+
 defaultParams = {  # pylint: disable=invalid-name
     "data.http_protocol": ("https", _make_validate_choice({"https", "http"})),
     "data.load": ("lazy", _make_validate_choice({"lazy", "eager"})),
     "data.index_origin": (0, _make_validate_choice({0, 1}, typeof=int)),
     "plot.backend": ("matplotlib", _make_validate_choice({"matplotlib", "bokeh"})),
-    "plot.bokeh.tools": (
-        "pan,box_zoom,wheel_zoom,box_select,lasso_select,undo,redo,reset,save,hover",
-        lambda x: x,
-    ),
-    "plot.bokeh.output_backend": ("webgl", _make_validate_choice({"canvas", "svg", "webgl"})),
-    "plot.bokeh.figure.dpi": (60, _validate_positive_int),
-    "plot.bokeh.figure.width": (500, _validate_positive_int),
-    "plot.bokeh.figure.height": (500, _validate_positive_int),
-    "plot.bokeh.show": (True, _validate_boolean),
-    "plot.matplotlib.constrained_layout": (True, _validate_boolean),
-    "plot.matplotlib.show": (False, _validate_boolean),
     "plot.max_subplots": (40, _validate_positive_int_or_none),
     "plot.point_estimate": (
         "mean",
         _make_validate_choice({"mean", "median", "mode"}, allow_none=True),
     ),
+    "plot.bokeh.bounds_x_range": ("auto", _validate_bokeh_bounds),
+    "plot.bokeh.bounds_y_range": ("auto", _validate_bokeh_bounds),
+    "plot.bokeh.figure.dpi": (60, _validate_positive_int),
+    "plot.bokeh.figure.height": (500, _validate_positive_int),
+    "plot.bokeh.figure.width": (500, _validate_positive_int),
+    "plot.bokeh.layout.order": (
+        "default",
+        _make_validate_choice_regex(
+            {"default", r"column", r"row", "square", "square_trimmed"}, {r"\d*column", r"\d*row"}
+        ),
+    ),
+    "plot.bokeh.layout.sizing_mode": (
+        "fixed",
+        _make_validate_choice(
+            {
+                "fixed",
+                "stretch_width",
+                "stretch_height",
+                "stretch_both",
+                "scale_width",
+                "scale_height",
+                "scale_both",
+            }
+        ),
+    ),
+    "plot.bokeh.layout.toolbar_location": (
+        "above",
+        _make_validate_choice({"above", "below", "left", "right"}, allow_none=True),
+    ),
+    "plot.bokeh.marker": ("Cross", _validate_bokeh_marker),
+    "plot.bokeh.output_backend": ("webgl", _make_validate_choice({"canvas", "svg", "webgl"})),
+    "plot.bokeh.show": (True, _validate_boolean),
+    "plot.bokeh.tools": (
+        "reset,pan,box_zoom,wheel_zoom,lasso_select,undo,save,hover",
+        lambda x: x,
+    ),
+    "plot.matplotlib.constrained_layout": (True, _validate_boolean),
+    "plot.matplotlib.show": (False, _validate_boolean),
     "stats.credible_interval": (0.94, _validate_probability),
-    "stats.information_criterion": ("waic", _make_validate_choice({"waic", "loo"})),
-    "stats.ic_scale": ("deviance", _make_validate_choice({"deviance", "log", "negative_log"})),
+    "stats.information_criterion": ("loo", _make_validate_choice({"waic", "loo"})),
+    "stats.ic_scale": ("log", _make_validate_choice({"deviance", "log", "negative_log"})),
 }
 
 
