@@ -12,19 +12,20 @@ import scipy.stats as st
 from scipy.optimize import minimize
 import xarray as xr
 
+from ..plots.plot_utils import _fast_kde, get_bins
 from ..data import convert_to_inference_data, convert_to_dataset, InferenceData, CoordSpec, DimSpec
-from ..plots.kdeplot import _fast_kde
-from ..plots.plot_utils import get_bins
-from .diagnostics import _multichain_statistics, _mc_error, ess, _circular_standard_deviation
+from .diagnostics import _multichain_statistics, _mc_error, ess
 from .stats_utils import (
     make_ufunc as _make_ufunc,
     wrap_xarray_ufunc as _wrap_xarray_ufunc,
     logsumexp as _logsumexp,
     ELPDData,
     stats_variance_2d as svar,
+    histogram,
+    _circular_standard_deviation,
+    get_log_likelihood as _get_log_likelihood,
 )
 from ..utils import _var_names, Numba, _numba_var
-from ..stats.stats_utils import histogram
 from ..rcparams import rcParams
 
 _log = logging.getLogger(__name__)
@@ -43,26 +44,21 @@ __all__ = [
 
 
 def compare(
-    dataset_dict,
-    ic=None,
-    method="BB-pseudo-BMA",
-    b_samples=1000,
-    alpha=1,
-    seed=None,
-    scale="deviance",
+    dataset_dict, ic=None, method="BB-pseudo-BMA", b_samples=1000, alpha=1, seed=None, scale=None
 ):
-    r"""Compare models based on WAIC or LOO cross-validation.
+    r"""Compare models based on PSIS-LOO `loo` or WAIC `waic` cross-validation.
 
-    WAIC is the widely applicable information criterion, and LOO is leave-one-out
-    (LOO) cross-validation. Read more theory here - in a paper by some of the
-    leading authorities on model selection - dx.doi.org/10.1111/1467-9868.00353
+    LOO is leave-one-out (PSIS-LOO `loo`) cross-validation and
+    WAIC is the widely applicable information criterion.
+    Read more theory here - in a paper by some of the leading authorities
+    on model selection dx.doi.org/10.1111/1467-9868.00353
 
     Parameters
     ----------
     dataset_dict : dict[str] -> InferenceData
         A dictionary of model names and InferenceData objects
     ic : str
-        Information Criterion (WAIC or LOO) used to compare models. Defaults to
+        Information Criterion (PSIS-LOO `loo` or WAIC `waic`) used to compare models. Defaults to
         ``rcParams["stats.information_criterion"]``.
     method : str
         Method used to estimate the weights for each model. Available options are:
@@ -88,21 +84,23 @@ def compare(
     scale : str
         Output scale for IC. Available options are:
 
-        - `deviance` : (default) -2 * (log-score)
-        - `log` : 1 * log-score (after Vehtari et al. (2017))
+        - `log` : (default) log-score (after Vehtari et al. (2017))
         - `negative_log` : -1 * (log-score)
+        - `deviance` : -2 * (log-score)
 
     Returns
     -------
     A DataFrame, ordered from best to worst model (measured by information criteria).
     The index reflects the key with which the models are passed to this function. The columns are:
     rank : The rank-order of the models. 0 is the best.
-    IC : Information Criteria (WAIC or LOO).
-        Smaller IC indicates higher out-of-sample predictive fit ("better" model). Default WAIC.
-        If `scale == log` higher IC indicates higher out-of-sample predictive fit ("better" model).
+    IC : Information Criteria (PSIS-LOO `loo` or WAIC `waic`).
+        Higher IC indicates higher out-of-sample predictive fit ("better" model). Default LOO.
+        If `scale` is `deviance` or `negative_log` smaller IC indicates
+        higher out-of-sample predictive fit ("better" model).
     pIC : Estimated effective number of parameters.
-    dIC : Relative difference between each IC (WAIC or LOO) and the lowest IC (WAIC or LOO).
-        It's always 0 for the top-ranked model.
+    dIC : Relative difference between each IC (PSIS-LOO `loo` or WAIC `waic`)
+          and the lowest IC (PSIS-LOO `loo` or WAIC `waic`).
+          The top-ranked model is always 0.
     weight: Relative weight for each model.
         This can be loosely interpreted as the probability of each model (among the compared model)
         given the data. By default the uncertainty in the weights estimation is considered using
@@ -137,7 +135,7 @@ def compare(
 
     """
     names = list(dataset_dict.keys())
-    scale = scale.lower()
+    scale = rcParams["stats.ic_scale"] if scale is None else scale.lower()
     if scale == "log":
         scale_value = 1
         ascending = False
@@ -149,25 +147,7 @@ def compare(
         ascending = True
 
     ic = rcParams["stats.information_criterion"] if ic is None else ic.lower()
-    if ic == "waic":
-        ic_func = waic
-        df_comp = pd.DataFrame(
-            index=names,
-            columns=[
-                "rank",
-                "waic",
-                "p_waic",
-                "d_waic",
-                "weight",
-                "se",
-                "dse",
-                "warning",
-                "waic_scale",
-            ],
-        )
-        scale_col = "waic_scale"
-
-    elif ic == "loo":
+    if ic == "loo":
         ic_func = loo
         df_comp = pd.DataFrame(
             index=names,
@@ -184,7 +164,23 @@ def compare(
             ],
         )
         scale_col = "loo_scale"
-
+    elif ic == "waic":
+        ic_func = waic
+        df_comp = pd.DataFrame(
+            index=names,
+            columns=[
+                "rank",
+                "waic",
+                "p_waic",
+                "d_waic",
+                "weight",
+                "se",
+                "dse",
+                "warning",
+                "waic_scale",
+            ],
+        )
+        scale_col = "waic_scale"
     else:
         raise NotImplementedError("The information criterion {} is not supported.".format(ic))
 
@@ -309,7 +305,7 @@ def _ic_matrix(ics, ic_i):
     return rows, cols, ic_i_val
 
 
-def hpd(ary, credible_interval=0.94, circular=False, multimodal=False):
+def hpd(ary, credible_interval=None, circular=False, multimodal=False, skipna=False):
     """
     Calculate highest posterior density (HPD) of array for given credible_interval.
 
@@ -328,6 +324,8 @@ def hpd(ary, credible_interval=0.94, circular=False, multimodal=False):
     multimodal : bool
         If true it may compute more than one hpd interval if the distribution is multimodal and the
         modes are well separated.
+    skipna : bool
+        If true ignores nan values when computing the hpd interval. Defaults to false.
 
     Returns
     -------
@@ -345,6 +343,12 @@ def hpd(ary, credible_interval=0.94, circular=False, multimodal=False):
            ...: data = np.random.normal(size=2000)
            ...: az.hpd(data, credible_interval=.68)
     """
+    if credible_interval is None:
+        credible_interval = rcParams["stats.credible_interval"]
+    else:
+        if not 1 >= credible_interval > 0:
+            raise ValueError("The value of credible_interval should be in the interval (0, 1]")
+
     if ary.ndim > 1:
         hpd_array = np.array(
             [
@@ -360,6 +364,9 @@ def hpd(ary, credible_interval=0.94, circular=False, multimodal=False):
         return hpd_array
 
     if multimodal:
+        if skipna:
+            ary = ary[~np.isnan(ary)]
+
         if ary.dtype.kind == "f":
             density, lower, upper = _fast_kde(ary)
             range_x = upper - lower
@@ -388,7 +395,10 @@ def hpd(ary, credible_interval=0.94, circular=False, multimodal=False):
         hpd_intervals = np.array(hpd_intervals)
 
     else:
-        ary = ary.copy()
+        if skipna:
+            nans = np.isnan(ary)
+            if not nans.all():
+                ary = ary[~nans]
         n = len(ary)
 
         if circular:
@@ -402,10 +412,7 @@ def hpd(ary, credible_interval=0.94, circular=False, multimodal=False):
         interval_width = ary[interval_idx_inc:] - ary[:n_intervals]
 
         if len(interval_width) == 0:
-            raise ValueError(
-                "Too few elements for interval calculation. "
-                "Check that credible_interval meets condition 0 =< credible_interval < 1"
-            )
+            raise ValueError("Too few elements for interval calculation. ")
 
         min_idx = np.argmin(interval_width)
         hdi_min = ary[min_idx]
@@ -422,7 +429,7 @@ def hpd(ary, credible_interval=0.94, circular=False, multimodal=False):
     return hpd_intervals
 
 
-def loo(data, pointwise=False, reff=None, scale="deviance"):
+def loo(data, pointwise=False, reff=None, scale=None):
     """Pareto-smoothed importance sampling leave-one-out cross-validation.
 
     Calculates leave-one-out (LOO) cross-validation for out of sample predictive model fit,
@@ -442,9 +449,9 @@ def loo(data, pointwise=False, reff=None, scale="deviance"):
     scale : str
         Output scale for loo. Available options are:
 
-        - `deviance` : (default) -2 * (log-score)
-        - `log` : 1 * log-score (after Vehtari et al. (2017))
-        - `negative_log` : -1 * (log-score)
+        - `log` : (default) log-score (after Vehtari et al. (2017))
+        - `negative_log` : -1 * log-score
+        - `deviance` : -2 * log-score
 
         A higher log-score (or a lower deviance) indicates a model with better predictive
         accuracy.
@@ -476,8 +483,8 @@ def loo(data, pointwise=False, reff=None, scale="deviance"):
            ...: az.loo(data)
 
     The custom print method can be seen here, printing only the relevant information and
-    with a specific organization. ``IC_loo`` stands for information criteria, which is the
-    `deviance` scale, the `log` (and `negative_log`) correspond to ``elpd`` (and ``-elpd``)
+    with a specific organization. ``deviance_loo`` stands for `deviance` scale,
+    the `log` (and `negative_log`) correspond to ``elpd`` (and ``-elpd``)
 
     .. ipython::
 
@@ -485,21 +492,19 @@ def loo(data, pointwise=False, reff=None, scale="deviance"):
 
     """
     inference_data = convert_to_inference_data(data)
-    if not hasattr(inference_data, "sample_stats"):
-        raise TypeError("Must be able to extract a sample_stats group from data.")
-    if "log_likelihood" not in inference_data.sample_stats:
-        raise TypeError("Data must include log_likelihood in sample_stats")
-    log_likelihood = inference_data.sample_stats.log_likelihood
+    log_likelihood = _get_log_likelihood(inference_data)
+
     log_likelihood = log_likelihood.stack(sample=("chain", "draw"))
     shape = log_likelihood.shape
     n_samples = shape[-1]
     n_data_points = np.product(shape[:-1])
+    scale = rcParams["stats.ic_scale"] if scale is None else scale.lower()
 
-    if scale.lower() == "deviance":
+    if scale == "deviance":
         scale_value = -2
-    elif scale.lower() == "log":
+    elif scale == "log":
         scale_value = 1
-    elif scale.lower() == "negative_log":
+    elif scale == "negative_log":
         scale_value = -1
     else:
         raise TypeError('Valid scale values are "deviance", "log", "negative_log"')
@@ -809,9 +814,10 @@ def summary(
     include_circ=None,
     stat_funcs=None,
     extend=True,
-    credible_interval=0.94,
+    credible_interval=None,
     order="C",
-    index_origin=0,
+    index_origin=None,
+    skipna=False,
     coords: Optional[CoordSpec] = None,
     dims: Optional[DimSpec] = None,
 ) -> Union[pd.DataFrame, xr.Dataset]:
@@ -843,15 +849,19 @@ def summary(
         The functions should be in the style of a ufunc and return a single number. For example,
         `np.mean`, or `scipy.stats.var` would both work.
     extend : boolean
-        If True, use the statistics returned by `stat_funcs` in addition to, rather than in place
-        of, the default statistics. This is only meaningful when `stat_funcs` is not None.
+        If True, use the statistics returned by ``stat_funcs`` in addition to, rather than in place
+        of, the default statistics. This is only meaningful when ``stat_funcs`` is not None.
     credible_interval : float, optional
-        Credible interval to plot. Defaults to 0.94. This is only meaningful when `stat_funcs` is
+        Credible interval to plot. Defaults to 0.94. This is only meaningful when ``stat_funcs`` is
         None.
     order : {"C", "F"}
         If fmt is "wide", use either C or F unpacking order. Defaults to C.
     index_origin : int
-        If fmt is "wide, select n-based indexing for multivariate parameters. Defaults to 0.
+        If fmt is "wide, select n-based indexing for multivariate parameters.
+        Defaults to rcParam data.index.origin, which is 0.
+    skipna : bool
+        If true ignores nan values when computing the summary statistics, it does not affect the
+        behaviour of the functions passed to ``stat_funcs``. Defaults to false.
     coords: Dict[str, List[Any]], optional
         Coordinates specification to be used if the ``fmt`` is ``'xarray'``.
     dims: Dict[str, List[str]], optional
@@ -905,6 +915,13 @@ def summary(
         extra_args["coords"] = coords
     if dims is not None:
         extra_args["dims"] = dims
+    if index_origin is None:
+        index_origin = rcParams["data.index_origin"]
+    if credible_interval is None:
+        credible_interval = rcParams["stats.credible_interval"]
+    else:
+        if not 1 >= credible_interval > 0:
+            raise ValueError("The value of credible_interval should be in the interval (0, 1]")
     posterior = convert_to_dataset(data, group="posterior", **extra_args)
     var_names = _var_names(var_names, posterior)
     posterior = posterior if var_names is None else posterior[var_names]
@@ -943,35 +960,38 @@ def summary(
                 extra_metric_names.append(stat_func.__name__)
 
     if extend and kind in ["all", "stats"]:
-        mean = posterior.mean(dim=("chain", "draw"))
+        mean = posterior.mean(dim=("chain", "draw"), skipna=skipna)
 
-        sd = posterior.std(dim=("chain", "draw"), ddof=1)
+        sd = posterior.std(dim=("chain", "draw"), ddof=1, skipna=skipna)
 
         hpd_lower, hpd_higher = xr.apply_ufunc(
             _make_ufunc(hpd, n_output=2),
             posterior,
-            kwargs=dict(credible_interval=credible_interval, multimodal=False),
+            kwargs=dict(credible_interval=credible_interval, multimodal=False, skipna=skipna),
             input_core_dims=(("chain", "draw"),),
             output_core_dims=tuple([] for _ in range(2)),
         )
 
     if include_circ:
+        nan_policy = "omit" if skipna else "propagate"
         circ_mean = xr.apply_ufunc(
             _make_ufunc(st.circmean),
             posterior,
-            kwargs=dict(high=np.pi, low=-np.pi),
+            kwargs=dict(high=np.pi, low=-np.pi, nan_policy=nan_policy),
             input_core_dims=(("chain", "draw"),),
         )
         _numba_flag = Numba.numba_flag
         func = None
         if _numba_flag:
             func = _circular_standard_deviation
+            kwargs_circ_std = dict(high=np.pi, low=-np.pi, skipna=skipna)
         else:
             func = st.circstd
+            kwargs_circ_std = dict(high=np.pi, low=-np.pi, nan_policy=nan_policy)
         circ_sd = xr.apply_ufunc(
             _make_ufunc(func),
             posterior,
-            kwargs=dict(high=np.pi, low=-np.pi),
+            kwargs=kwargs_circ_std,
             input_core_dims=(("chain", "draw"),),
         )
 
@@ -985,7 +1005,7 @@ def summary(
         circ_hpd_lower, circ_hpd_higher = xr.apply_ufunc(
             _make_ufunc(hpd, n_output=2),
             posterior,
-            kwargs=dict(credible_interval=credible_interval, circular=True),
+            kwargs=dict(credible_interval=credible_interval, circular=True, skipna=skipna),
             input_core_dims=(("chain", "draw"),),
             output_core_dims=tuple([] for _ in range(2)),
         )
@@ -1099,7 +1119,7 @@ def summary(
     return summary_df
 
 
-def waic(data, pointwise=False, scale="deviance"):
+def waic(data, pointwise=False, scale=None):
     """Calculate the widely available information criterion.
 
     Also calculates the WAIC's standard error and the effective number of
@@ -1116,11 +1136,11 @@ def waic(data, pointwise=False, scale="deviance"):
         if True the pointwise predictive accuracy will be returned.
         Default False
     scale : str
-        Output scale for loo. Available options are:
+        Output scale for WAIC. Available options are:
 
-        - `deviance` : (default) -2 * (log-score)
-        - `log` : 1 * log-score
-        - `negative_log` : -1 * (log-score)
+        - `log` : (default) log-score
+        - `negative_log` : -1 * log-score
+        - `deviance` : -2 * log-score
 
         A higher log-score (or a lower deviance) indicates a model with better predictive
         accuracy.
@@ -1151,24 +1171,18 @@ def waic(data, pointwise=False, scale="deviance"):
            ...: az.waic(data, pointwise=True)
 
     The custom print method can be seen here, printing only the relevant information and
-    with a specific organization. ``IC_loo`` stands for information criteria, which is the
-    `deviance` scale, the `log` (and `negative_log`) correspond to ``elpd`` (and ``-elpd``)
+    with a specific organization. ``deviance_loo`` stands for `deviance` scale,
+    the `log` (and `negative_log`) correspond to ``elpd`` (and ``-elpd``)
     """
     inference_data = convert_to_inference_data(data)
-    for group in ("sample_stats",):
-        if not hasattr(inference_data, group):
-            raise TypeError(
-                "Must be able to extract a {group} group from data.".format(group=group)
-            )
-    if "log_likelihood" not in inference_data.sample_stats:
-        raise TypeError("Data must include log_likelihood in sample_stats")
-    log_likelihood = inference_data.sample_stats.log_likelihood
+    log_likelihood = _get_log_likelihood(inference_data)
+    scale = rcParams["stats.ic_scale"] if scale is None else scale.lower()
 
-    if scale.lower() == "deviance":
+    if scale == "deviance":
         scale_value = -2
-    elif scale.lower() == "log":
+    elif scale == "log":
         scale_value = 1
-    elif scale.lower() == "negative_log":
+    elif scale == "negative_log":
         scale_value = -1
     else:
         raise TypeError('Valid scale values are "deviance", "log", "negative_log"')
@@ -1250,7 +1264,7 @@ def waic(data, pointwise=False, scale="deviance"):
 
 
 def loo_pit(idata=None, *, y=None, y_hat=None, log_weights=None):
-    """Compute leave one out (LOO) probability integral transform (PIT) values.
+    """Compute leave one out (PSIS-LOO) probability integral transform (PIT) values.
 
     Parameters
     ----------
@@ -1294,6 +1308,7 @@ def loo_pit(idata=None, *, y=None, y_hat=None, log_weights=None):
            ...: az.loo_pit(idata=data, y=T**2, y_hat=T_hat**2)
 
     """
+    y_str = ""
     if idata is not None and not isinstance(idata, InferenceData):
         raise ValueError("idata must be of type InferenceData or None")
 
@@ -1310,6 +1325,7 @@ def loo_pit(idata=None, *, y=None, y_hat=None, log_weights=None):
         elif y_hat is None:
             raise ValueError("y_hat cannot be None if y is not a str")
         if isinstance(y, str):
+            y_str = y
             y = idata.observed_data[y].values
         elif not isinstance(y, (np.ndarray, xr.DataArray)):
             raise ValueError("y must be of types array, DataArray or str, not {}".format(type(y)))
@@ -1320,7 +1336,14 @@ def loo_pit(idata=None, *, y=None, y_hat=None, log_weights=None):
                 "y_hat must be of types array, DataArray or str, not {}".format(type(y_hat))
             )
         if log_weights is None:
-            log_likelihood = idata.sample_stats.log_likelihood.stack(sample=("chain", "draw"))
+            if y_str:
+                try:
+                    log_likelihood = _get_log_likelihood(idata, var_name=y)
+                except TypeError:
+                    log_likelihood = _get_log_likelihood(idata)
+            else:
+                log_likelihood = _get_log_likelihood(idata)
+            log_likelihood = log_likelihood.stack(sample=("chain", "draw"))
             posterior = convert_to_dataset(idata, group="posterior")
             n_chains = len(posterior.chain)
             n_samples = len(log_likelihood.sample)
