@@ -2,13 +2,14 @@
 from collections.abc import Sequence
 import logging
 import warnings
+from copy import copy as _copy, deepcopy as _deepcopy
 
 import numpy as np
 import pandas as pd
 from scipy.fftpack import next_fast_len
 from scipy.stats.mstats import mquantiles
 from xarray import apply_ufunc
-from ..utils import conditional_jit
+from ..utils import conditional_jit, conditional_vect
 
 _log = logging.getLogger(__name__)
 
@@ -386,6 +387,33 @@ def not_valid(ary, check_nan=True, check_shape=True, nan_kwargs=None, shape_kwar
     return nan_error | chain_error | draw_error
 
 
+def get_log_likelihood(idata, var_name=None):
+    """Retrieve the log likelihood dataarray of a given variable."""
+    if hasattr(idata, "sample_stats") and hasattr(idata.sample_stats, "log_likelihood"):
+        warnings.warn(
+            "Storing the log_likelihood in sample_stats groups will be deprecated",
+            PendingDeprecationWarning,
+        )
+        return idata.sample_stats.log_likelihood
+    if not hasattr(idata, "log_likelihood"):
+        raise TypeError("log likelihood not found in inference data object")
+    if var_name is None:
+        var_names = list(idata.log_likelihood.data_vars)
+        if "lp" in var_names:
+            var_names.remove("lp")
+        if len(var_names) > 1:
+            raise TypeError(
+                "Found several log likelihood arrays {}, var_name cannot be None".format(var_names)
+            )
+        return idata.log_likelihood[var_names[0]]
+    else:
+        try:
+            log_likelihood = idata.log_likelihood[var_name]
+        except KeyError:
+            raise TypeError("No log likelihood data named {} found".format(var_name))
+        return log_likelihood
+
+
 BASE_FMT = """Computed from {{n_samples}} by {{n_points}} log-likelihood matrix
 
 {{0:{0}}} Estimate       SE
@@ -400,7 +428,7 @@ Pareto k diagnostic values:
    (0.7, 1]   (bad)      {{4:{0}d}} {{8:6.1f}}%
    (1, Inf)   (very bad) {{5:{0}d}} {{9:6.1f}}%
 """
-SCALE_DICT = {"deviance": "IC", "log": "elpd", "negative_log": "-elpd"}
+SCALE_DICT = {"deviance": "deviance", "log": "elpd", "negative_log": "-elpd"}
 
 
 class ELPDData(pd.Series):  # pylint: disable=too-many-ancestors
@@ -410,7 +438,7 @@ class ELPDData(pd.Series):  # pylint: disable=too-many-ancestors
         """Print elpd data in a user friendly way."""
         kind = self.index[0]
 
-        if kind not in ("waic", "loo"):
+        if kind not in ("loo", "waic"):
             raise ValueError("Invalid ELPDData object")
 
         scale_str = SCALE_DICT[self["{}_scale".format(kind)]]
@@ -442,6 +470,16 @@ class ELPDData(pd.Series):  # pylint: disable=too-many-ancestors
         """Alias to ``__str__``."""
         return self.__str__()
 
+    def copy(self, deep=True):
+        """Perform a pandas deep copy of the ELPDData plus a copy of the stored data."""
+        copied_obj = pd.Series.copy(self)
+        for key in copied_obj.keys():
+            if deep:
+                copied_obj[key] = _deepcopy(copied_obj[key])
+            else:
+                copied_obj[key] = _copy(copied_obj[key])
+        return ELPDData(copied_obj)
+
 
 @conditional_jit
 def stats_variance_1d(data, ddof=0):
@@ -495,3 +533,31 @@ def histogram(data, bins, range_hist=None):
     hist, bin_edges = np.histogram(data, bins=bins, range=range_hist)
     hist_dens = hist / (hist.sum() * np.diff(bin_edges))
     return hist, hist_dens, bin_edges
+
+
+@conditional_vect
+def _sqrt(a_a, b_b):
+    return (a_a + b_b) ** 0.5
+
+
+def _circfunc(samples, high, low, skipna):
+    samples = np.asarray(samples)
+    if skipna:
+        samples = samples[~np.isnan(samples)]
+    if samples.size == 0:
+        return np.nan
+    return _angle(samples, low, high, np.pi)
+
+
+@conditional_vect
+def _angle(samples, low, high, p_i=np.pi):
+    ang = (samples - low) * 2.0 * p_i / (high - low)
+    return ang
+
+
+def _circular_standard_deviation(samples, high=2 * np.pi, low=0, skipna=False, axis=None):
+    ang = _circfunc(samples, high, low, skipna)
+    s_s = np.sin(ang).mean(axis=axis)
+    c_c = np.cos(ang).mean(axis=axis)
+    r_r = np.hypot(s_s, c_c)
+    return ((high - low) / 2.0 / np.pi) * np.sqrt(-2 * np.log(r_r))
