@@ -6,10 +6,11 @@ import os
 import pickle
 import sys
 import logging
+from typing import Dict, List, Tuple, Union
 import pytest
 import numpy as np
 
-from ..data import from_dict
+from ..data import from_dict, InferenceData
 
 
 _log = logging.getLogger(__name__)
@@ -157,16 +158,24 @@ def multidim_models():
     return Models()
 
 
-def check_multiple_attrs(test_dict, parent):
+def check_multiple_attrs(
+    test_dict: Dict[str, List[str]], parent: InferenceData
+) -> List[Union[str, Tuple[str, str]]]:
     """Perform multiple hasattr checks on InferenceData objects.
 
     It is thought to first check if the parent object contains a given dataset,
     and then (if present) check the attributes of the dataset.
 
-    Args
-    ----
-    test_dict: dict
-        Its structure should be `{dataset1_name: [var1, var2], dataset2_name: [var]}`
+    Given the ouput of the function, all missmatches between expectation and reality can
+    be retrieved: a single string indicates a group mismatch and a tuple of strings
+    ``(group, var)`` indicates a mismatch in the variable ``var`` of ``group``.
+
+    Parameters
+    ----------
+    test_dict: dict of {str : list of str}
+        Its structure should be `{dataset1_name: [var1, var2], dataset2_name: [var]}`.
+        A ``~`` at the beggining of a dataset or variable name indicates the name NOT
+        being present must be asserted.
     parent: InferenceData
         InferenceData object on which to check the attributes.
 
@@ -176,13 +185,36 @@ def check_multiple_attrs(test_dict, parent):
         List containing the failed checks. It will contain either the dataset_name or a
         tuple (dataset_name, var) for all non present attributes.
 
+    Examples
+    --------
+    The output below indicates that ``posterior`` group was expected but not found, and
+    variables ``a`` and ``b``:
+
+        ["posterior", ("prior", "a"), ("prior", "b")]
+
+    Another example could be the following:
+
+        [("posterior", "a"), "~observed_data", ("sample_stats", "~log_likelihood")]
+
+    In this case, the output indicates that variable ``a`` was not found in ``posterior``
+    as it was expected, however, in the other two cases, the preceding ``~`` (kept from the
+    input negation notation) indicates that ``observed_data`` group should not be present
+    but was found in the InferenceData and that ``log_likelihood`` variable was found
+    in ``sample_stats``, also against what was expected.
+
     """
     failed_attrs = []
     for dataset_name, attributes in test_dict.items():
-        if hasattr(parent, dataset_name):
+        if dataset_name.startswith("~"):
+            if hasattr(parent, dataset_name[1:]):
+                failed_attrs.append(dataset_name)
+        elif hasattr(parent, dataset_name):
             dataset = getattr(parent, dataset_name)
             for attribute in attributes:
-                if not hasattr(dataset, attribute):
+                if attribute.startswith("~"):
+                    if hasattr(dataset, attribute[1:]):
+                        failed_attrs.append((dataset_name, attribute))
+                elif not hasattr(dataset, attribute):
                     failed_attrs.append((dataset_name, attribute))
         else:
             failed_attrs.append(dataset_name)
@@ -324,7 +356,8 @@ def numpyro_schools_model(data, draws, chains):
     # This block lets the posterior be pickled
     mcmc.sampler._sample_fn = None  # pylint: disable=protected-access
     mcmc.sampler._init_fn = None  # pylint: disable=protected-access
-    mcmc.sampler._constrain_fn = None  # pylint: disable=protected-access
+    mcmc.sampler._postprocess_fn = None  # pylint: disable=protected-access
+    mcmc.sampler._potential_fn = None  # pylint: disable=protected-access
     mcmc._cache = {}  # pylint: disable=protected-access
     return mcmc
 
@@ -434,7 +467,7 @@ def pystan_noncentered_schools(data, draws, chains):
         }
     """
     if pystan_version() == 2:
-        import pystan
+        import pystan  # pylint: disable=import-error
 
         stan_model = pystan.StanModel(model_code=schools_code)
         fit = stan_model.sampling(
@@ -537,7 +570,12 @@ def pystan_version():
 
     """
     try:
-        import pystan
+        import pystan  # pylint: disable=import-error
     except ImportError:
         import stan as pystan  # pylint: disable=import-error
     return int(pystan.__version__[0])
+
+
+def test_precompile_models(eight_schools_params, draws, chains):
+    """Precompile model files."""
+    load_cached_models(eight_schools_params, draws, chains)
